from django.db import models
from apis_core.apis_vocabularies.models import LabelType
#from metainfo.models import TempEntityClass
#from reversion import revisions as reversion
import reversion

import unicodedata


@reversion.register()
class Label(models.Model):
    label = models.CharField(
        max_length=255, blank=True, help_text="The entities label or name.")
    isoCode_639_3 = models.CharField(
        max_length=3, blank=True, null=True,
        help_text="The ISO 639-3 (or 2) code for the label's language.",
        verbose_name='ISO Code', default='deu')
    label_type = models.ForeignKey(LabelType, blank=True, null=True,
                                   on_delete=models.SET_NULL)
    temp_entity = models.ForeignKey("apis_metainfo.TempEntityClass", on_delete=models.CASCADE)

    def get_web_object(self):
        result = {
            'relation_pk': self.pk,
            'label': self.label,
            'isoCode_639_3': self.isoCode_639_3,
            'label_type': self.label_type.name}
        return result

    def save(self, *args, **kwargs):
        if self.label != unicodedata.normalize('NFC', self.label):    #secure correct unicode encoding
            self.label = unicodedata.normalize('NFC', self.label)
        super(Label, self).save(*args, **kwargs)
        return self



#from autocomplete_light import shortcuts as al
from dal import autocomplete
import requests, json
from .models import Label


class LabelAutocomplete(autocomplete.Select2QuerySetView):
    autocomplete_template = 'apis_labels/label_al.html'
    widget_attrs = {

    }
    attrs = {
        'data-autocomplete-minimum-characters': 3,
        'placeholder': u'Please start typing language names (in English!) to get suggestions'}

    # choices = [{'ref_name':'Ghotuo', 'isoID':'aaa'},{'ref_name':'Alumu-Tesu', 'isoID':'aab'}]

    def choices_for_request(self):
        choices = []
        root = "http://iso639.eos.arz.oeaw.ac.at/api/fuzzysearch/?language="
        parameter = "&format=json"
        q = self.request.GET.get('q')
        url = root + q + parameter
        r = requests.get(url)
        try:
            response = r.json()
            results = response["results"]
            for x in results:
                choices.append({"ref_name": x["ref_name"], "isoID": x["isoID"]})
            return choices
        except:
            choices = [{"ref_name": """Sorry, something went wrong.
			Please refere to http://www-01.sil.org/iso639-3/codes.asp 
			to find the matching iso639-3 code.
			We apologize for your unceonvenicen."""}]
            return choices


#al.register(LabelAutocomplete)



# -*- coding: utf-8 -*-
from dal import autocomplete
from .fields import ListSelect2, Select2Multiple
from django import forms
from django.urls import reverse_lazy
from crispy_forms.helper import FormHelper
from crispy_forms.layout import Submit
from crispy_forms.layout import Layout
from crispy_forms.bootstrap import Accordion, AccordionGroup
from django.forms import ModelMultipleChoiceField
from django.utils.translation import ugettext_lazy as _
from django.urls import reverse
from django.db.models.fields import BLANK_CHOICE_DASH
from django.contrib.contenttypes.models import ContentType
from django.core.validators import URLValidator
from django.conf import settings

from .models import Person, Place, Institution, Event, Work
from apis_core.apis_vocabularies.models import TextType
from apis_core.apis_metainfo.models import Text, Uri

from apis_core.helper_functions.RDFparsers import GenericRDFParser

if 'apis_highlighter' in settings.INSTALLED_APPS:
    from apis_highlighter.models import AnnotationProject


class SearchForm(forms.Form):
    search = forms.CharField(label='Search')

    @property
    def helper(self):
        helper = FormHelper()
        helper.form_id = 'searchForm'
        helper.form_tag = False
        helper.add_input(Submit('fieldn', 'search'))
        helper.form_method = 'GET'
        return helper


def get_entities_form(entity):
    class GenericEntitiesForm(forms.ModelForm):
        class Meta:
            model = ContentType.objects.get(
                app_label='apis_entities', model=entity.lower()).model_class()
            exclude = ['start_date', 'end_date', 'text', 'source']

        def __init__(self, *args, **kwargs):
            super(GenericEntitiesForm, self).__init__(*args, **kwargs)
            self.helper = FormHelper()
            self.helper.form_class = entity.title()+'Form'
            self.helper.form_tag = False
            self.helper.help_text_inline = True
            acc_grp1 = AccordionGroup('Metadata {}'.format(entity.title()))
            acc_grp2 = AccordionGroup(
                        'MetaInfo',
                        'collection',
                        'references',
                        'notes',
                        'review')
            attrs = {'data-placeholder': 'Type to get suggestions',
                     'data-minimum-input-length': 3,
                     'data-html': True}
            for f in self.fields.keys():
                if type(self.fields[f]) == ModelMultipleChoiceField:
                    v_name_p = str(self.fields[f].queryset.model.__name__)
                    if ContentType.objects.get(app_label__in=[
                        'apis_entities',
                        'apis_metainfo',
                        'apis_relations',
                        'apis_vocabularies',
                        'apis_labels'
                    ], model=v_name_p.lower()).app_label.lower() == 'apis_vocabularies':
                        self.fields[f].widget = Select2Multiple(
                            url=reverse(
                                'apis:apis_vocabularies:generic_vocabularies_autocomplete',
                                kwargs={
                                    'vocab': v_name_p.lower(),
                                    'direct': 'normal'
                                    }
                                ),
                            attrs=attrs)
                        if self.instance:
                            res = []
                            try:
                                for x in getattr(self.instance, f).all():
                                    res.append((x.pk, x.name))
                            except ValueError:
                                pass
                            self.fields[f].choices = res
                            self.fields[f].initial = res
                if f not in acc_grp2:
                    acc_grp1.append(f)

            self.helper.layout = Layout(
                Accordion(
                    acc_grp1,
                    acc_grp2
                    )
            )
            self.fields['status'].required = False
            self.fields['collection'].required = False
            self.fields['start_date_written'].required = False
            self.fields['end_date_written'].required = False
    return GenericEntitiesForm


class GenericEntitiesStanbolForm(forms.Form):
    def save(self, *args, **kwargs):
        cd = self.cleaned_data
        entity = GenericRDFParser(cd['entity'], self.entity.title()).get_or_create()
        return entity

    def __init__(self, entity, *args, **kwargs):
        attrs = {'data-placeholder': 'Type to get suggestions',
                 'data-minimum-input-length': 3,
                 'data-html': True,
                 'style': 'width: auto'}
        ent_merge_pk = kwargs.pop('ent_merge_pk', False)
        super(GenericEntitiesStanbolForm, self).__init__(*args, **kwargs)
        self.entity = entity
        self.helper = FormHelper()
        form_kwargs = {'entity': entity}
        url = reverse(
            'apis:apis_entities:generic_entities_autocomplete', args=[entity.title(), 'remove']
        )
        label = 'Create {} from reference resources'.format(entity.title())
        button_label = 'Create'
        if ent_merge_pk:
            form_kwargs['ent_merge_pk'] = ent_merge_pk
            url = reverse('apis:apis_entities:generic_entities_autocomplete', args=[entity.title()])
            label = 'Search for {0} in reference resources or db'.format(entity.title())
            button_label = 'Merge'
        self.helper.form_action = reverse(
            'apis:apis_entities:generic_entities_stanbol_create', kwargs=form_kwargs
        )
        self.helper.add_input(Submit('submit', button_label))
        self.fields['entity'] = autocomplete.Select2ListCreateChoiceField(
                label=label,
                widget=ListSelect2(
                    url=url,
                    attrs=attrs),
                validators=[URLValidator])


class FullTextForm(forms.Form):

    def save(self, entity):
        cd = self.cleaned_data
        text = None
        for f in cd.keys():
            text_type = TextType.objects.get(pk=f.split('_')[1])
            print(text_type)
            text = Text.objects.filter(tempentityclass=entity, kind=text_type)
            if text.count() == 1:
                text = text[0]
                text.text = cd[f]
                text.save()
            elif text.count() == 0:
                text = Text(text=cd[f], kind=text_type)
                text.save()
                entity.text.add(text)
        return text

    def __init__(self, *args, **kwargs):
        if 'entity' in kwargs.keys():
            entity = kwargs.pop('entity', None)
        else:
            entity = None
        if 'instance' in kwargs.keys():
            instance = kwargs.pop('instance', None)
        else:
            instance = None
        super(FullTextForm, self).__init__(*args, **kwargs)
        self.helper = FormHelper()
        self.helper.form_class = 'FullTextForm'
        self.helper.form_tag = False
        self.helper.help_text_inline = True
        collections = []
        if instance:
            for i in instance.collection.all():
                collections.append(i)
        try:
            if len(collections) > 0:
                q = TextType.objects.filter(entity__iexact=entity, collections__in=collections)
            else:
                q = TextType.objects.filter(entity__iexact=entity)
            for txt in q:
                self.fields['text_'+str(txt.pk)] = forms.CharField(
                    label=txt.name,
                    help_text=txt.description,
                    required=False,
                    widget=forms.Textarea)
            if instance:
                for t in instance.text.all():
                    if 'text_'+str(t.kind.pk) in self.fields.keys():
                        self.fields['text_'+str(t.kind.pk)].initial = t.text
        except:
            pass


class PersonResolveUriForm(forms.Form):
    # person = forms.CharField(label=False, widget=al.TextWidget('PersonAutocomplete'))
    person = forms.CharField(label=False)
    person_uri = forms.CharField(required=False, widget=forms.HiddenInput())

    def save(self, site_instance, instance=None, commit=True):
        cd = self.cleaned_data
        if cd['person'].startswith('http'):
            uri = Uri.objects.create(uri=cd['person'], entity=site_instance)
        else:
            uri = Uri.objects.create(uri=cd['person_uri'], entity=site_instance)
        return uri

    def __init__(self, *args, **kwargs):
        entity_type = kwargs.pop('entity_type', False)
        self.request = kwargs.pop('request', False)
        super(PersonResolveUriForm, self).__init__(*args, **kwargs)
        self.helper = FormHelper()
        self.helper.form_tag = False

    def clean(self):
        cleaned_data = super(PersonResolveUriForm, self).clean()
        if Uri.objects.filter(uri=cleaned_data['person_uri']).exists():
            self.add_error('person', 'This Person has already been added to the DB.')
        elif cleaned_data['person'].startswith('http'):
            if Uri.objects.filter(uri=cleaned_data['person']).exists():
                self.add_error('person', 'This URI has already been added to the DB.')


class NetworkVizFilterForm(forms.Form):
    ann_include_all = forms.BooleanField(
        required=False, label='Include general relations',
        help_text="""Not all relations are connected to an annotation.\
        If checked relations that are not attached to an annotation are include.\
        This setting is only used when an Annotation project is specified."""
    )
    start_date = forms.CharField(
        label='Start date',
        required=False,
        widget=forms.TextInput(
            attrs={
                'data-provide': 'datepicker',
                'data-date-format': 'dd.mm.yyyy'
            }
        )
    )
    end_date = forms.CharField(
        label='End date',
        required=False,
        widget=forms.TextInput(
            attrs={
                'data-provide': 'datepicker',
                'data-date-format': 'dd.mm.yyyy'
            }
        )
    )

    def __init__(self, *args, **kwargs):
        rel_attrs = {
            'data-placeholder': 'Type to get suggestions',
            'data-minimum-input-length': 0,
            'data-html': True
        }
        attrs = {
            'data-placeholder': 'Type to get suggestions',
            'data-minimum-input-length': 0,
            'data-html': True
        }
        super(NetworkVizFilterForm, self).__init__(*args, **kwargs)
        self.fields['select_relation'] = forms.ChoiceField(
            label='Relation type',
            choices=list(('-'.join(x.name.split()), x.name) for x in ContentType.objects.filter(
                app_label='apis_relations')
            ),
            help_text="Include only relations related to this annotation project \
            (See the include general relations checkbox)")
        self.fields['select_relation'].initial = ('person-place', 'person place')
        self.fields['search_source'] = autocomplete.Select2ListCreateChoiceField(
            label='Search source',
            widget=ListSelect2(
                url=reverse(
                    'apis:apis_entities:generic_network_entities_autocomplete',
                    kwargs={
                        'entity': 'person'
                    }
                ),
                attrs=attrs
            )
        )
        self.fields['search_target'] = autocomplete.Select2ListCreateChoiceField(
            label='Search target',
            widget=ListSelect2(
                url=reverse(
                    'apis:apis_entities:generic_network_entities_autocomplete',
                    kwargs={
                        'entity': 'place'
                    }
                ),
                attrs=attrs
            )
        )
        self.fields['select_kind'] = autocomplete.Select2ListCreateChoiceField(
            label='Select kind',
            widget=ListSelect2(
                url=reverse(
                    'apis:apis_vocabularies:generic_vocabularies_autocomplete',
                    kwargs={
                        'vocab': 'personplacerelation',
                        'direct': 'normal'
                    }
                ),
                attrs=rel_attrs
            )
        )
        if 'apis_highlighter' in settings.INSTALLED_APPS:
            self.fields['annotation_proj'] = forms.ChoiceField(
                label='Annotation Project',
                choices=BLANK_CHOICE_DASH + list(
                    (x.pk, x.name) for x in AnnotationProject.objects.all()),
                required=False,
                help_text="Include only relations related to this annotation project \
                (See the include general relations checkbox)")
        self.helper = FormHelper()
        self.helper.form_class = 'FilterNodesForm'
        self.helper.form_action = reverse('apis:NetJson-list')
        self.helper.add_input(Submit('Submit', 'Add nodes'))
        self.order_fields(('select_relation', 'ann_include_all',
                           'annotation_proj', 'search_source', 'select_kind', 'search_target'))


class GenericFilterFormHelper(FormHelper):
    def __init__(self, *args, **kwargs):
        super(GenericFilterFormHelper, self).__init__(*args, **kwargs)
        self.helper = FormHelper()
        self.form_class = 'genericFilterForm'
        self.form_method = 'GET'
        self.add_input(Submit('Filter', 'Filter'))



from django.test import TestCase

from .models import Text, Source
from apis_core.apis_vocabularies.models import TextType
from apis_highlighter.models import Annotation


class TextModelTestCase(TestCase):
    text = 'Hallo wir sind heute in Wien. Wie geht es euch?'
    source_name = 'testsource'
    ann_start = 5
    ann_stop = 10


    @classmethod
    def setUpTestData(cls):
        # Set up data for the whole TestCase
        cls.source = Source.objects.create(pubinfo=cls.source_name)

    def test_init(self):
        Text()

    def test_add_to_text(self):
        txt = Text.objects.create(text=self.text, source=self.source)
        ann = Annotation.objects.create(start=self.ann_start, end=self.ann_stop, text=txt)
        anno_orig = txt.text[ann.start:ann.end]
        txt.text = txt.text[:self.ann_start-3]+ 'test....'+txt.text[self.ann_start-2:]
        txt.save()
        ann2 = Annotation.objects.get(pk=ann.pk)
        anno_change = txt.text[ann2.start:ann2.end]
        self.assertEqual(anno_orig, anno_change)

    def test_delete_text(self):
        txt = Text.objects.create(text=self.text, source=self.source)
        ann = Annotation.objects.create(start=self.ann_start, end=self.ann_stop, text=txt)
        anno_orig = txt.text[ann.start:ann.end]
        print('test2: '+txt.text)
        txt.text = txt.text[:self.ann_start-4]+ txt.text[self.ann_start-2:]
        txt.save()
        ann2 = Annotation.objects.get(pk=ann.pk)
        anno_change = txt.text[ann2.start:ann2.end]
        self.assertEqual(anno_orig, anno_change)

    def test_anno_in_delete(self):
        txt = Text.objects.create(text=self.text, source=self.source)
        ann = Annotation.objects.create(start=self.ann_start, end=self.ann_stop, text=txt)
        anno_orig = txt.text[ann.start:ann.end]
        txt.text = txt.text[:self.ann_start - 2] + txt.text[self.ann_stop - 2:]
        txt.save()
        with self.assertRaises(Annotation.DoesNotExist):
            Annotation.objects.get(pk=ann.pk)



# Generated by Django 2.1.2 on 2018-10-03 13:10

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('apis_metainfo', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='VocabNames',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255)),
            ],
        ),
        migrations.CreateModel(
            name='VocabsBaseClass',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255, verbose_name='Name')),
                ('description', models.TextField(blank=True, help_text='Brief description of the used term.')),
                ('status', models.CharField(choices=[('rej', 'rejected'), ('ac', 'accepted'), ('can', 'candidate'), ('del', 'deleted')], default='can', max_length=4)),
            ],
        ),
        migrations.CreateModel(
            name='VocabsUri',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('uri', models.URLField()),
                ('domain', models.CharField(blank=True, max_length=255)),
                ('rdf_link', models.URLField(blank=True)),
                ('loaded', models.BooleanField(default=False)),
                ('loaded_time', models.DateTimeField(blank=True, null=True)),
            ],
        ),
        migrations.CreateModel(
            name='CollectionType',
            fields=[
                ('vocabsbaseclass_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='apis_vocabularies.VocabsBaseClass')),
            ],
            bases=('apis_vocabularies.vocabsbaseclass',),
        ),
        migrations.CreateModel(
            name='EventType',
            fields=[
                ('vocabsbaseclass_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='apis_vocabularies.VocabsBaseClass')),
            ],
            bases=('apis_vocabularies.vocabsbaseclass',),
        ),
        migrations.CreateModel(
            name='InstitutionType',
            fields=[
                ('vocabsbaseclass_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='apis_vocabularies.VocabsBaseClass')),
            ],
            bases=('apis_vocabularies.vocabsbaseclass',),
        ),
        migrations.CreateModel(
            name='LabelType',
            fields=[
                ('vocabsbaseclass_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='apis_vocabularies.VocabsBaseClass')),
            ],
            bases=('apis_vocabularies.vocabsbaseclass',),
        ),
        migrations.CreateModel(
            name='PlaceType',
            fields=[
                ('vocabsbaseclass_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='apis_vocabularies.VocabsBaseClass')),
            ],
            bases=('apis_vocabularies.vocabsbaseclass',),
        ),
        migrations.CreateModel(
            name='ProfessionType',
            fields=[
                ('vocabsbaseclass_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='apis_vocabularies.VocabsBaseClass')),
            ],
            bases=('apis_vocabularies.vocabsbaseclass',),
        ),
        migrations.CreateModel(
            name='RelationBaseClass',
            fields=[
                ('vocabsbaseclass_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='apis_vocabularies.VocabsBaseClass')),
                ('name_reverse', models.CharField(blank=True, help_text='Inverse relation like: "is sub-class of" vs. "is super-class of".', max_length=255, verbose_name='Name reverse')),
            ],
            bases=('apis_vocabularies.vocabsbaseclass',),
        ),
        migrations.CreateModel(
            name='TextType',
            fields=[
                ('vocabsbaseclass_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='apis_vocabularies.VocabsBaseClass')),
                ('entity', models.CharField(max_length=255)),
                ('collections', models.ManyToManyField(blank=True, to='apis_metainfo.Collection')),
            ],
            bases=('apis_vocabularies.vocabsbaseclass',),
        ),
        migrations.CreateModel(
            name='Title',
            fields=[
                ('vocabsbaseclass_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='apis_vocabularies.VocabsBaseClass')),
                ('abbreviation', models.CharField(blank=True, max_length=10)),
            ],
            bases=('apis_vocabularies.vocabsbaseclass',),
        ),
        migrations.CreateModel(
            name='WorkType',
            fields=[
                ('vocabsbaseclass_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='apis_vocabularies.VocabsBaseClass')),
            ],
            bases=('apis_vocabularies.vocabsbaseclass',),
        ),
        migrations.AddField(
            model_name='vocabsuri',
            name='vocab',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='apis_vocabularies.VocabsBaseClass'),
        ),
        migrations.AddField(
            model_name='vocabsbaseclass',
            name='parent_class',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='apis_vocabularies.VocabsBaseClass'),
        ),
        migrations.AddField(
            model_name='vocabsbaseclass',
            name='userAdded',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to=settings.AUTH_USER_MODEL),
        ),
        migrations.AddField(
            model_name='vocabsbaseclass',
            name='vocab_name',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='apis_vocabularies.VocabNames'),
        ),
        migrations.CreateModel(
            name='EventEventRelation',
            fields=[
                ('relationbaseclass_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='apis_vocabularies.RelationBaseClass')),
            ],
            bases=('apis_vocabularies.relationbaseclass',),
        ),
        migrations.CreateModel(
            name='EventWorkRelation',
            fields=[
                ('relationbaseclass_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='apis_vocabularies.RelationBaseClass')),
            ],
            bases=('apis_vocabularies.relationbaseclass',),
        ),
        migrations.CreateModel(
            name='InstitutionEventRelation',
            fields=[
                ('relationbaseclass_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='apis_vocabularies.RelationBaseClass')),
            ],
            bases=('apis_vocabularies.relationbaseclass',),
        ),
        migrations.CreateModel(
            name='InstitutionInstitutionRelation',
            fields=[
                ('relationbaseclass_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='apis_vocabularies.RelationBaseClass')),
            ],
            bases=('apis_vocabularies.relationbaseclass',),
        ),
        migrations.CreateModel(
            name='InstitutionPlaceRelation',
            fields=[
                ('relationbaseclass_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='apis_vocabularies.RelationBaseClass')),
            ],
            bases=('apis_vocabularies.relationbaseclass',),
        ),
        migrations.CreateModel(
            name='InstitutionWorkRelation',
            fields=[
                ('relationbaseclass_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='apis_vocabularies.RelationBaseClass')),
            ],
            bases=('apis_vocabularies.relationbaseclass',),
        ),
        migrations.CreateModel(
            name='PersonEventRelation',
            fields=[
                ('relationbaseclass_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='apis_vocabularies.RelationBaseClass')),
            ],
            bases=('apis_vocabularies.relationbaseclass',),
        ),
        migrations.CreateModel(
            name='PersonInstitutionRelation',
            fields=[
                ('relationbaseclass_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='apis_vocabularies.RelationBaseClass')),
            ],
            bases=('apis_vocabularies.relationbaseclass',),
        ),
        migrations.CreateModel(
            name='PersonPersonRelation',
            fields=[
                ('relationbaseclass_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='apis_vocabularies.RelationBaseClass')),
            ],
            bases=('apis_vocabularies.relationbaseclass',),
        ),
        migrations.CreateModel(
            name='PersonPlaceRelation',
            fields=[
                ('relationbaseclass_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='apis_vocabularies.RelationBaseClass')),
            ],
            bases=('apis_vocabularies.relationbaseclass',),
        ),
        migrations.CreateModel(
            name='PersonWorkRelation',
            fields=[
                ('relationbaseclass_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='apis_vocabularies.RelationBaseClass')),
            ],
            bases=('apis_vocabularies.relationbaseclass',),
        ),
        migrations.CreateModel(
            name='PlaceEventRelation',
            fields=[
                ('relationbaseclass_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='apis_vocabularies.RelationBaseClass')),
            ],
            bases=('apis_vocabularies.relationbaseclass',),
        ),
        migrations.CreateModel(
            name='PlacePlaceRelation',
            fields=[
                ('relationbaseclass_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='apis_vocabularies.RelationBaseClass')),
            ],
            bases=('apis_vocabularies.relationbaseclass',),
        ),
        migrations.CreateModel(
            name='PlaceWorkRelation',
            fields=[
                ('relationbaseclass_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='apis_vocabularies.RelationBaseClass')),
            ],
            bases=('apis_vocabularies.relationbaseclass',),
        ),
        migrations.CreateModel(
            name='WorkWorkRelation',
            fields=[
                ('relationbaseclass_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='apis_vocabularies.RelationBaseClass')),
            ],
            bases=('apis_vocabularies.relationbaseclass',),
        ),
    ]



from django.test import TestCase

from entities.forms import PersonHighlighterForm
from metainfo.models import Text


class PersonHighlighterFormTestCase(TestCase):

    @classmethod
    def setUpTestData(cls):
        # Set up data for the whole TestCase
        txt = Text.objects.create(text='hgjhg kjgjhgj')
        cls.form = PersonHighlighterForm({
            'person': 'test',
            'person_uri': 'http://d-nb.info/gnd/118566512',
            'HL_start': 4,
            'HL_end': 6,
            'HL_text_id': 'htxt_' + str(txt.pk)
            })
        cls.form.is_valid()

    def test_init(self):
        PersonHighlighterForm()

    def test_valid_data(self):
        f = PersonHighlighterForm()
        print(f)
        self.assertTrue(self.form.is_valid())

    def test_save_form(self):
        x = self.form.save()
        self.assertEqual(x.name, 'Kreisky')
        self.assertEqual(x.first_name, 'Bruno')
        self.assertEqual(x.start_date_written, '22.01.1911')
        print(x)



from django.contrib import admin

from .models import Source, Collection, Text, Uri


admin.site.register(Source)
admin.site.register(Collection)
admin.site.register(Text)
admin.site.register(Uri)






from django.conf.urls import url

from .autocomplete3 import GenericEntitiesAutocomplete, GenericNetworkEntitiesAutocomplete
from .views import ReversionCompareView
from .views2 import GenericEntitiesCreateStanbolView
from . import views, views2, detail_views

app_name = 'apis_entities'

urlpatterns = [
    url(r'^entity/(?P<entity>[a-z]+)/(?P<pk>[0-9]+)/edit$',
        views2.GenericEntitiesEditView.as_view(),
        name='generic_entities_edit_view'),
    url(r'^entity/(?P<entity>[a-z]+)/(?P<pk>[0-9]+)/detail$',
        detail_views.GenericEntitiesDetailView.as_view(),
        name='generic_entities_detail_view'),
    url(r'^entity/(?P<entity>[a-z]+)/create$', views2.GenericEntitiesCreateView.as_view(),
        name='generic_entities_create_view'),
    url(r'^entity/(?P<entity>[a-z]+)/(?P<pk>[0-9]+)/delete$',
        views2.GenericEntitiesDeleteView.as_view(),
        name='generic_entities_delete_view'),
    url(r'^entity/(?P<entity>[a-z]+)/list/$',
        views.GenericListViewNew.as_view(), name='generic_entities_list'),
    url(r'^autocomplete/createstanbol/(?P<entity>[a-zA-Z0-9-]+)/$',
        GenericEntitiesCreateStanbolView.as_view(),
        name='generic_entities_stanbol_create'),
    url(r'^autocomplete/createstanbol/(?P<entity>[a-zA-Z0-9-]+)/(?P<ent_merge_pk>[0-9]+)/$',
        GenericEntitiesCreateStanbolView.as_view(),
        name='generic_entities_stanbol_create'),
    url(r'^autocomplete/(?P<entity>[a-zA-Z0-9-]+)/$',
        GenericEntitiesAutocomplete.as_view(),
        name='generic_entities_autocomplete'),
    url(r'^autocomplete/(?P<entity>[a-zA-Z0-9-]+)/(?P<db_include>[a-z]+)/$',
        GenericEntitiesAutocomplete.as_view(),
        name='generic_entities_autocomplete'),
    url(r'^autocomplete-network/(?P<entity>[a-zA-Z0-9-]+)/$',
        GenericNetworkEntitiesAutocomplete.as_view(),
        name='generic_network_entities_autocomplete'),
    url(r'^detail/work/(?P<pk>[0-9]+)$',
        detail_views.WorkDetailView.as_view(), name='work_detail'),
    url(r'^place/geojson/$', views.getGeoJson, name='getGeoJson'),
    url(r'^place/geojson/list/$', views.getGeoJsonList, name='getGeoJsonList'),
    url(r'^place/network/list/$', views.getNetJsonList, name='getNetJsonList'),
    url(r'^resolve/place/(?P<pk>[0-9]+)/(?P<uri>.+)$',
        views.resolve_ambigue_place, name='resolve_ambigue_place'),
    url(r'^maps/birthdeath/$', views.birth_death_map, name='birth_death_map'),
    url(r'^networks/relation_place/$', views.pers_place_netw, name='pers_place_netw'),
    url(r'^networks/relation_institution/$', views.pers_inst_netw, name='pers_inst_netw'),
    url(r'^networks/generic/$', views.generic_network_viz, name='generic_network_viz'),
    url(
        r'^compare/(?P<app>[a-z]+)/(?P<kind>[a-z]+)/(?P<pk>\d+)$', ReversionCompareView.as_view()
    ),
]



from django.db import models
#from reversion import revisions as reversion
import reversion
from django.db.models import Q
import operator
import pdb

from apis_core.apis_entities.models import Person, Place, Institution, Event, Work
from apis_core.apis_metainfo.models import TempEntityClass
from apis_core.apis_vocabularies.models import (PersonPlaceRelation, PersonPersonRelation,
    PersonInstitutionRelation, PersonEventRelation, PersonWorkRelation,
    InstitutionInstitutionRelation, InstitutionPlaceRelation,
    InstitutionEventRelation, PlacePlaceRelation, PlaceEventRelation,
    PlaceWorkRelation, EventEventRelation, EventWorkRelation,
    WorkWorkRelation, InstitutionWorkRelation)


#######################################################################
#
# Custom Managers
#
#######################################################################


class AnnotationRelationLinkManager(models.Manager):
    """Manager used to retrieve only those relations that are highlighted in the texts.
    Reads out the ``annotation_project`` and ``users_show_highlighter`` session variables and provides a filter.
    Needs a :class:`django.request` object in order to read out the ``session`` variable.

    *Example:*
    ::

        relation = PersonPlace.objects.filter(related_place='Wien').filter_ann_project(request=request)

    Returns only those relations that are connected with an annotation that fits the session variables or are not
    connected to any annotation at all.
    """
    def filter_ann_proj(self, request=None, ann_proj=1, include_all=True):
        """The filter function provided by the manager class.

        :param request: `django.request` object
        :return: queryset that contains only objects that are shown in the highlighted text or those not connected
            to an annotation at all.
        """
        users_show = None
        if request:
            ann_proj = request.session.get('annotation_project', 1)
            if not ann_proj:
                ann_proj = 1
            users_show = request.session.get('users_show_highlighter', None)
        query = Q(annotation__annotation_project_id=ann_proj)
        qs = super(AnnotationRelationLinkManager, self).get_queryset()
        if users_show is not None:
            query.add(Q(annotation__user_added_id__in=users_show), Q.AND)
        if include_all:
            query.add(Q(annotation__annotation_project__isnull=True), Q.OR)
        return qs.filter(query)


#######################################################################
#
# Person - ... - Relation
#
#######################################################################


@reversion.register(follow=['tempentityclass_ptr'])
class PersonPerson(TempEntityClass):
    """Defines and describes a relation between a Person and another Person

    :param int relation_type: Foreign Key to :class:`vocabularies.models.PersonPersonRelation`
    :param int related_personA: Foreign Key to :class:`entities.models.Person`
    :param int related_personB: Foreign Key to :class:`entities.models.Person`
    """

    relation_type = models.ForeignKey(PersonPersonRelation, blank=True,
                                      null=True, on_delete=models.SET_NULL)
    related_personA = models.ForeignKey(
        Person, blank=True, null=True, related_name="related_personA",
        on_delete=models.CASCADE)
    related_personB = models.ForeignKey(
        Person, blank=True, null=True, related_name="related_personB",
        on_delete=models.CASCADE)
    objects = models.Manager()
    annotation_links = AnnotationRelationLinkManager()

    def __str__(self):
        return "{} ({}) {}".format(self.related_personA, self.relation_type, self.related_personB)

    def get_web_object(self):
        """Used in some html views.

        :return: Dict with object properties
        """
        if self.related_personA.first_name is None:
            self.related_personA.first_name = '-'
        if self.related_personB.first_name is None:
            self.related_personB.first_name = '-'
        result = {
            'relation_pk': self.pk,
            'relation_type': self.relation_type.name,
            'related_personA': self.related_personA.name+', '+self.related_personA.first_name,
            'related_personB': self.related_personB.name+', '+self.related_personB.first_name,
            'start_date': self.start_date_written,
            'end_date': self.end_date_written}
        return result

    def get_table_dict(self, entity):
        """
        Function that returns dict used in relation tables.

        :param entity: :class:`entities.models.Person` instance that is the starting point of the table.
        :return: dict
        """
        if self.related_personA == entity:
            rel_pers = self.related_personB
            rel_type = self.relation_type.name
        elif self.related_personB == entity:
            rel_pers = self.related_personA
            rel_type = self.relation_type.name_reverse
        result = {
            'pk': self.pk,
            'relation_type': rel_type,
            'related_person': rel_pers,
            'start_date': self.start_date,
            'end_date': self.end_date}
        return result


@reversion.register(follow=['tempentityclass_ptr'])
class PersonPlace(TempEntityClass):
    """Defines and describes a relation between a Person and a Place

    :param int relation_type: Foreign Key to :class:`vocabularies.models.PersonPlaceRelation`
    :param int related_person: Foreign Key to :class:`entities.models.Person`
    :param int related_place: Foreign Key to :class:`entities.models.Place`
    """

    relation_type = models.ForeignKey(PersonPlaceRelation, blank=True,
                                      null=True, on_delete=models.SET_NULL)
    related_person = models.ForeignKey(
        Person, blank=True, null=True, on_delete=models.CASCADE)
    related_place = models.ForeignKey(
        Place, blank=True, null=True, on_delete=models.CASCADE)
    objects = models.Manager()
    annotation_links = AnnotationRelationLinkManager()

    def __str__(self):
        return "{} ({}) {}".format(self.related_person, self.relation_type, self.related_place)

    def get_web_object(self):
        """Used in some html views.

        :return: Dict with object properties
        """

        if self.related_person.first_name is None:
            self.related_person.first_name = '-'
        result = {
            'relation_pk': self.pk,
            'relation_type': self.relation_type.name,
            'related_person': self.related_person.name+', '+self.related_person.first_name,
            'related_place': self.related_place.name,
            'start_date': self.start_date_written,
            'end_date': self.end_date_written}
        return result


@reversion.register(follow=['tempentityclass_ptr'])
class PersonInstitution(TempEntityClass):
    """ Defines and describes a relation between a Person and a Institution

    :param int relation_type: Foreign Key to :class:`vocabularies.models.PersonInstitutionRelation`
    :param int related_person: Foreign Key to :class:`entities.models.Person`
    :param int related_institution: Foreign Key to :class:`entities.models.Institution`
    """

    relation_type = models.ForeignKey(PersonInstitutionRelation, blank=True,
                                      null=True, on_delete=models.SET_NULL)
    related_person = models.ForeignKey(
        Person, blank=True, null=True, on_delete=models.CASCADE)
    related_institution = models.ForeignKey(
        Institution, blank=True, null=True, on_delete=models.CASCADE)
    objects = models.Manager()
    annotation_links = AnnotationRelationLinkManager()

    def __str__(self):
        return "{} ({}) {}".format(
            self.related_person, self.relation_type, self.related_institution)

    def get_web_object(self):

        if self.related_person.first_name is None:
            self.related_person.first_name = '-'
        result = {
            'relation_pk': self.pk,
            'relation_type': self.relation_type.name,
            'related_person': self.related_person.name+', '+self.related_person.first_name,
            'related_institution': self.related_institution.name,
            'start_date': self.start_date_written,
            'end_date': self.end_date_written}
        return result


@reversion.register(follow=['tempentityclass_ptr'])
class PersonEvent(TempEntityClass):
    """ Defines and describes a relation bewteen a Person and an Event

    :param int relation_type: Foreign Key to :class:`vocabularies.models.PersonEventRelation`
    :param int related_person: Foreign Key to :class:`entities.models.Person`
    :param int related_event: Foreign Key to :class:`entities.models.Event`
    """

    relation_type = models.ForeignKey(PersonEventRelation, blank=True,
                                      null=True, on_delete=models.SET_NULL)
    related_person = models.ForeignKey(
        Person, blank=True, null=True, on_delete=models.CASCADE)
    related_event = models.ForeignKey(
        Event, blank=True, null=True, on_delete=models.CASCADE)
    objects = models.Manager()
    annotation_links = AnnotationRelationLinkManager()

    def __str__(self):
        return "{} ({}) {}".format(self.related_person, self.relation_type, self.related_event)

    def get_web_object(self):

        if self.related_person.first_name is None:
            self.related_person.first_name = '-'
        result = {
            'relation_pk': self.pk,
            'relation_type': self.relation_type.name,
            'related_person': self.related_person.name+', '+self.related_person.first_name,
            'related_event': self.related_event.name,
            'start_date': self.start_date_written,
            'end_date': self.end_date_written}
        return result


@reversion.register(follow=['tempentityclass_ptr'])
class PersonWork(TempEntityClass):
    """ Defines and describes a relation between a Person and a Work

    :param int relation_type: Foreign Key to :class:`vocabularies.models.PersonWorkRelation`
    :param int related_person: Foreign Key to :class:`entities.models.Person`
    :param int related_work: Foreign Key to :class:`entities.models.Work`
    """

    relation_type = models.ForeignKey(PersonWorkRelation, blank=True, null=True,
                                      on_delete=models.SET_NULL)
    related_person = models.ForeignKey(
        Person, blank=True, null=True, on_delete=models.CASCADE)
    related_work = models.ForeignKey(
        Work, blank=True, null=True, on_delete=models.CASCADE)
    objects = models.Manager()
    annotation_links = AnnotationRelationLinkManager()

    def __str__(self):
        return "{} ({}) {}".format(self.related_person, self.relation_type, self.related_work)

    def get_web_object(self):

        if self.related_person.first_name is None:
            self.related_person.first_name = '-'
        result = {
            'relation_pk': self.pk,
            'relation_type': self.relation_type.name,
            'related_person': self.related_person.name+', '+self.related_person.first_name,
            'related_work': self.related_work.name,
            'start_date': self.start_date_written,
            'end_date': self.end_date_written}
        return result


#######################################################################
#
#   Institution - ... - Relation
#
#######################################################################


@reversion.register(follow=['tempentityclass_ptr'])
class InstitutionInstitution(TempEntityClass):
    """ Defines and describes a relation between two Institutions

    :param int relation_type: Foreign Key to :class:`vocabularies.models.InstitutionInstitutionRelation`
    :param int related_institutionA: Foreign Key to :class:`entities.models.Institution`
    :param int related_institutionB: Foreign Key to :class:`entities.models.Institution`
    """

    relation_type = models.ForeignKey(InstitutionInstitutionRelation,
                                      blank=True, null=True,
                                      on_delete=models.SET_NULL)
    related_institutionA = models.ForeignKey(
        Institution, blank=True, null=True, related_name="related_institutionA",
        on_delete=models.CASCADE)
    related_institutionB = models.ForeignKey(
        Institution, blank=True, null=True, related_name="related_institutionB",
        on_delete=models.CASCADE)
    objects = models.Manager()
    annotation_links = AnnotationRelationLinkManager()

    def __str__(self):
        return "{} ({}) {}".format(
            self.related_institutionA, self.relation_type, self.related_institutionB)

    def get_web_object(self):
        result = {
            'relation_pk': self.pk,
            'relation_type': self.relation_type.name,
            'related_institutionA': self.related_institutionA.name,
            'related_institutionB': self.related_institutionB.name,
            'start_date': self.start_date_written,
            'end_date': self.end_date_written}
        return result

    def get_table_dict(self, entity):
        if self.related_institutionA == entity:
            rel_inst = self.related_institutionB
            rel_type = self.relation_type.name
        elif self.related_institutionB == entity:
            rel_inst = self.related_institutionA
            rel_type = self.relation_type.name_reverse
        result = {
            'relation_pk': self.pk,
            'relation_type': rel_type,
            'related_institution': rel_inst,
            'start_date': self.start_date,
            'end_date': self.end_date}
        return result


@reversion.register(follow=['tempentityclass_ptr'])
class InstitutionPlace(TempEntityClass):
    """Describes a relation bewteen an Institution  and a Place

    :param int relation_type: Foreign Key to :class:`vocabularies.models.InstitutionPlaceRelation`
    :param int related_institution: Foreign Key to :class:`entities.models.Institution`
    :param int related_place: Foreign Key to :class:`entities.models.Place`
    """

    relation_type = models.ForeignKey(
        InstitutionPlaceRelation, blank=True, null=True,
        on_delete=models.SET_NULL)
    related_institution = models.ForeignKey(
        Institution, blank=True, null=True, on_delete=models.CASCADE)
    related_place = models.ForeignKey(
        Place, blank=True, null=True, on_delete=models.CASCADE)
    objects = models.Manager()
    annotation_links = AnnotationRelationLinkManager()

    def __str__(self):
        return "{} ({}) {}".format(
            self.related_institution, self.relation_type, self.related_place)

    def get_web_object(self):
        result = {
            'relation_pk': self.pk,
            'relation_type': self.relation_type.name,
            'related_institution': self.related_institution.name,
            'related_place': self.related_place.name,
            'start_date': self.start_date_written,
            'end_date': self.end_date_written}
        return result


@reversion.register(follow=['tempentityclass_ptr'])
class InstitutionEvent(TempEntityClass):
    """Describes a relation bewteen an Institution and an Event

    :param int relation_type: Foreign Key to :class:`vocabularies.models.InstitutionEventRelation`
    :param int related_institution: Foreign Key to :class:`entities.models.Institution`
    :param int related_event: Foreign Key to :class:`entities.models.Event`
    """

    relation_type = models.ForeignKey(InstitutionEventRelation, blank=True,
                                      null=True, on_delete=models.SET_NULL)
    related_institution = models.ForeignKey(
        Institution, blank=True, null=True, on_delete=models.CASCADE)
    related_event = models.ForeignKey(
        Event, blank=True, null=True, on_delete=models.CASCADE)
    objects = models.Manager()
    annotation_links = AnnotationRelationLinkManager()

    def __str__(self):
        return "{} ({}) {}".format(
            self.related_institution, self.relation_type, self.related_event)

    def get_web_object(self):
        result = {
            'relation_pk': self.pk,
            'relation_type': self.relation_type.name,
            'related_institution': self.related_institution.name,
            'related_event': self.related_event.name,
            'start_date': self.start_date_written,
            'end_date': self.end_date_written}
        return result


@reversion.register(follow=['tempentityclass_ptr'])
class InstitutionWork(TempEntityClass):
    """Describes a relation bewteen an Institution and a Work

    :param int relation_type: Foreign Key to :class:`vocabularies.models.InstitutionWorkRelation`
    :param int related_institution: Foreign Key to :class:`entities.models.Institution`
    :param int related_work: Foreign Key to :class:`entities.models.Work`
    """

    relation_type = models.ForeignKey(InstitutionWorkRelation, blank=True,
                                      null=True, on_delete=models.SET_NULL)
    related_institution = models.ForeignKey(
        Institution, blank=True, null=True, on_delete=models.CASCADE)
    related_work = models.ForeignKey(
        Work, blank=True, null=True, on_delete=models.CASCADE)
    objects = models.Manager()
    annotation_links = AnnotationRelationLinkManager()

    def __str__(self):
        return "{} ({}) {}".format(
            self.related_institution, self.relation_type, self.related_work)

    def get_web_object(self):
        result = {
            'relation_pk': self.pk,
            'relation_type': self.relation_type.name,
            'related_institution': self.related_institution.name,
            'related_work': self.related_work.name,
            'start_date': self.start_date_written,
            'end_date': self.end_date_written}
        return result



#######################################################################
#
#   Place - ... - Relation
#
#######################################################################


@reversion.register(follow=['tempentityclass_ptr'])
class PlacePlace(TempEntityClass):
    """Describes a relation bewteen an Place  and a Place

    :param int relation_type: Foreign Key to :class:`vocabularies.models.PlacePlaceRelation`
    :param int related_placeA: Foreign Key to :class:`entities.models.Place`
    :param int related_placeB: Foreign Key to :class:`entities.models.Place`
    """

    relation_type = models.ForeignKey(PlacePlaceRelation, blank=True, null=True,
                                      on_delete=models.SET_NULL)
    related_placeA = models.ForeignKey(
        Place, blank=True, null=True, related_name="related_placeA",
        on_delete=models.CASCADE)
    related_placeB = models.ForeignKey(
        Place, blank=True, null=True, related_name="related_placeB",
        on_delete=models.CASCADE)
    objects = models.Manager()
    annotation_links = AnnotationRelationLinkManager()

    def __str__(self):
        return "{} ({}) {}".format(self.related_placeA, self.relation_type, self.related_placeB)

    def get_table_dict(self, entity):
        """Dict for the tabels in the html view

        :param entity: Object of type :class:`entities.models.Place`; Used to determine which Place is the main antity
            and which one the related.
        :return:
        """
        if self.related_placeA == entity:
            rel_place = self.related_placeB
            rel_type = self.relation_type.name
        elif self.related_placeB == entity:
            rel_place = self.related_placeA
            rel_type = self.relation_type.name_reverse
        result = {
            'relation_pk': self.pk,
            'relation_type': rel_type,
            'related_place': rel_place,
            'start_date': self.start_date,
            'end_date': self.end_date}
        return result

    def get_web_object(self):
        """Used in some html views.

        :return: Dict with object properties
        """
        result = {
            'relation_pk': self.pk,
            'relation_type': self.relation_type.name,
            'related_placeA': self.related_placeA.name,
            'related_placeB': self.related_placeB.name,
            'start_date': self.start_date_written,
            'end_date': self.end_date_written}
        return result


@reversion.register(follow=['tempentityclass_ptr'])
class PlaceEvent(TempEntityClass):
    """Describes a relation between an Place and an Event

    :param int relation_type: Foreign Key to :class:`vocabularies.models.PlaceEventRelation`
    :param int related_place: Foreign Key to :class:`entities.models.Place`
    :param int related_event: Foreign Key to :class:`entities.models.Event`
    """

    relation_type = models.ForeignKey(PlaceEventRelation, blank=True, null=True,
                                      on_delete=models.SET_NULL)
    related_place = models.ForeignKey(
        Place, blank=True, null=True, on_delete=models.CASCADE)
    related_event = models.ForeignKey(
        Event, blank=True, null=True, on_delete=models.CASCADE)
    objects = models.Manager()
    annotation_links = AnnotationRelationLinkManager()

    def __str__(self):
        return "{} ({}) {}".format(
            self.related_place, self.relation_type, self.related_event)

    def get_web_object(self):
        """Function that returns a dict that is used in html views.

        :return: dict of attributes
        """
        result = {
            'relation_pk': self.pk,
            'relation_type': self.relation_type.name,
            'related_place': self.related_place.name,
            'related_event': self.related_event.name,
            'start_date': self.start_date_written,
            'end_date': self.end_date_written}
        return result


@reversion.register(follow=['tempentityclass_ptr'])
class PlaceWork(TempEntityClass):
    """Describes a relation between an Place and a Work

    :param int relation_type: Foreign Key to :class:`vocabularies.models.PlaceWorkRelation`
    :param int related_place: Foreign Key to :class:`entities.models.Place`
    :param int related_Work: Foreign Key to :class:`entities.models.Work`
    """

    relation_type = models.ForeignKey(PlaceWorkRelation, blank=True, null=True,
                                      on_delete=models.SET_NULL)
    related_place = models.ForeignKey(
        Place, blank=True, null=True, on_delete=models.CASCADE)
    related_work = models.ForeignKey(
        Work, blank=True, null=True, on_delete=models.CASCADE)
    objects = models.Manager()
    annotation_links = AnnotationRelationLinkManager()

    def __str__(self):
        return "{} ({}) {}".format(
            self.related_place, self.relation_type, self.related_work)

    def get_web_object(self):
        """Function that returns a dict that is used in html views.

        :return: dict of attributes
        """
        result = {
            'relation_pk': self.pk,
            'relation_type': self.relation_type.name,
            'related_place': self.related_place.name,
            'related_work': self.related_work.name,
            'start_date': self.start_date_written,
            'end_date': self.end_date_written}
        return result


#######################################################################
#
#   Event - ... - Relation
#
#######################################################################


@reversion.register(follow=['tempentityclass_ptr'])
class EventEvent(TempEntityClass):
    """Describes a relation between an Event and an Event

    :param int relation_type: Foreign Key to :class:`vocabularies.models.EventEventRelation`
    :param int related_eventA: Foreign Key to :class:`entities.models.Event`
    :param int related_eventB: Foreign Key to :class:`entities.models.Event`
    """

    relation_type = models.ForeignKey(EventEventRelation, blank=True, null=True,
                                      on_delete=models.SET_NULL)
    related_eventA = models.ForeignKey(
        Event, blank=True, null=True, related_name="related_eventA",
        on_delete=models.CASCADE)
    related_eventB = models.ForeignKey(
        Event, blank=True, null=True, related_name="related_eventB",
        on_delete=models.CASCADE)
    objects = models.Manager()
    annotation_links = AnnotationRelationLinkManager()

    def __str__(self):
        return "{} ({}) {}".format(
            self.related_event, self.relation_type, self.related_event)


@reversion.register(follow=['tempentityclass_ptr'])
class EventWork(TempEntityClass):
    """Describes a relation between an Event and a Work

    :param int relation_type: Foreign Key to :class:`vocabularies.models.EventWorkRelation`
    :param int related_event: Foreign Key to :class:`entities.models.Event`
    :param int related_work: Foreign Key to :class:`entities.models.Work`
    """

    relation_type = models.ForeignKey(EventWorkRelation, blank=True, null=True,
                                      on_delete=models.SET_NULL)
    related_event = models.ForeignKey(
        Event, blank=True, null=True, on_delete=models.CASCADE)
    related_work = models.ForeignKey(
        Work, blank=True, null=True, on_delete=models.CASCADE)
    objects = models.Manager()
    annotation_links = AnnotationRelationLinkManager()

    def __str__(self):
        return "{} ({}) {}".format(
            self.related_event, self.relation_type, self.related_work)

    def get_web_object(self):
        """Function that returns a dict that is used in html views.

        :return: dict of attributes
        """
        result = {
            'relation_pk': self.pk,
            'relation_type': self.relation_type.name,
            'related_event': self.related_event.name,
            'related_work': self.related_work.name,
            'start_date': self.start_date_written,
            'end_date': self.end_date_written}
        return result


#######################################################################
#
#   Event - ... - Relation
#
#######################################################################


@reversion.register(follow=['tempentityclass_ptr'])
class WorkWork(TempEntityClass):
    """Describes a relation between an Work and a Work

    :param int relation_type: Foreign Key to :class:`vocabularies.models.WorkWorkRelation`
    :param int related_workA: Foreign Key to :class:`entities.models.Work`
    :param int related_workB: Foreign Key to :class:`entities.models.Work`
    """

    relation_type = models.ForeignKey(WorkWorkRelation, blank=True, null=True,
                                      on_delete=models.SET_NULL)
    related_workA = models.ForeignKey(
        Work, blank=True, null=True, related_name="related_workA",
        on_delete=models.CASCADE)
    related_workB = models.ForeignKey(
        Work, blank=True, null=True, related_name="related_workB",
        on_delete=models.CASCADE)
    objects = models.Manager()
    annotation_links = AnnotationRelationLinkManager()

    def get_table_dict(self, entity):
        """Dict for the tabels in the html view

        :param entity: Object of type :class:`entities.models.Place`; Used to determine which Place is the main antity
            and which one the related.
        :return:
        """
        if self.related_workA == entity:
            rel_work = self.related_workB
            rel_type = self.relation_type.name
        elif self.related_workB == entity:
            rel_work = self.related_workA
            rel_type = self.relation_type.name_reverse
        result = {
            'relation_pk': self.pk,
            'relation_type': rel_type,
            'related_work': rel_work,
            'start_date': self.start_date,
            'end_date': self.end_date}
        return result

    def get_web_object(self):
        """Used in some html views.

        :return: Dict with object properties
        """
        result = {
            'relation_pk': self.pk,
            'relation_type': self.relation_type.name,
            'related_workA': self.related_workA.name,
            'related_workB': self.related_workB.name,
            'start_date': self.start_date_written,
            'end_date': self.end_date_written}
        return result

    def __str__(self):
        return "{} ({}) {}".format(self.related_workA, self.relation_type, self.related_workB)



from django.conf.urls import url

from . tei_ac import TeiEntAc

app_name = 'apis_tei'

urlpatterns = [
    url(
        r'^autocomplete/(?P<entity>[a-zA-Z0-9-]+)/$',
        TeiEntAc.as_view(),
        name='generic_entities_autocomplete'
    ),
]



from django.urls import reverse_lazy
from rest_framework import serializers
from .models import Institution, Person, Place, Event, Work
import re


class BaseEntitySerializer(serializers.HyperlinkedModelSerializer):
    uri_set = serializers.HyperlinkedIdentityField(
        view_name="apis:uri-detail",
        lookup_field="pk"
    )
    collection = serializers.HyperlinkedIdentityField(
        view_name="apis:collection-detail",
        lookup_field="pk"
    )
    text = serializers.HyperlinkedIdentityField(
        view_name="apis:text-detail",
        lookup_field="pk"
    )


class InstitutionSerializer(BaseEntitySerializer):
    url = serializers.HyperlinkedIdentityField(
        view_name="apis:institution-detail",
        lookup_field="pk"
    )

    kind = serializers.HyperlinkedIdentityField(
        view_name="apis:institutiontype-detail",
        lookup_field="pk"
    )

    class Meta:
        model = Institution
        fields = ('url', 'id', 'name', 'uri_set', 'kind', 'collection', 'text')


class PersonSerializer(BaseEntitySerializer):

    url = serializers.HyperlinkedIdentityField(
        view_name="apis:person-detail",
        lookup_field="pk"
    )

    class Meta:
        model = Person
        fields = (
            'url', 'id', 'name', 'first_name', 'uri_set', 'profession', 'collection', 'text'
        )


class PlaceSerializer(BaseEntitySerializer):

    url = serializers.HyperlinkedIdentityField(
        view_name="apis:place-detail",
        lookup_field="pk"
    )

    kind = serializers.HyperlinkedIdentityField(
        view_name="apis:placetype-detail",
        lookup_field="pk"
    )

    class Meta:
        model = Place
        fields = (
            'url', 'id', 'name', 'uri_set', 'collection', 'text', 'kind', 'lng', 'lat'
        )


class EventSerializer(BaseEntitySerializer):

    url = serializers.HyperlinkedIdentityField(
        view_name="apis:event-detail",
        lookup_field="pk"
    )

    kind = serializers.HyperlinkedIdentityField(
        view_name="apis:eventtype-detail",
        lookup_field="pk"
    )

    class Meta:
        model = Event
        fields = (
            'url', 'id', 'name', 'uri_set', 'collection', 'text', 'kind'
        )


class WorkSerializer(BaseEntitySerializer):

    url = serializers.HyperlinkedIdentityField(
        view_name="apis:work-detail",
        lookup_field="pk"
    )

    kind = serializers.HyperlinkedIdentityField(
        view_name="apis:worktype-detail",
        lookup_field="pk"
    )

    class Meta:
        model = Work
        fields = (
            'url', 'id', 'name', 'uri_set', 'collection', 'text', 'kind'
        )


class GeoJsonSerializer(serializers.BaseSerializer):

    def to_representation(self, obj):
        p_pk = self.context.get('p_pk')
        short = False
        url_r = reverse_lazy(
            'apis:apis_entities:resolve_ambigue_place',
            kwargs={'pk': str(p_pk), 'uri': obj['id'][7:]}
        )
        long = False
        if 'http://www.w3.org/2003/01/geo/wgs84_pos#long' in obj.keys():
            long = float(obj['http://www.w3.org/2003/01/geo/wgs84_pos#long'][0]['value'])
            lat = float(obj['http://www.w3.org/2003/01/geo/wgs84_pos#lat'][0]['value'])
        elif 'long' in obj.keys():
            long = float(obj['long'][0]['value'])
            lat = float(obj['lat'][0]['value'])
            short = True
        if long:
            popup = ''
            for k in obj.keys():
                if k == 'id' or k == 'long' or k == 'lat':
                    continue
                if not short or k.startswith('http'):
                    title = k.split('#')[-1]
                else:
                    title = k
                popup += '<b>{}:</b> {}<br/>'.format(title, obj[k][0]['value'])
            r = {"geometry": {
                    "type": "Point",
                    "coordinates": [long, lat]
                },
                "type": "Feature",
                "properties": {
                    "popupContent": """{}
                    <b>Geonames:</b> <a href='{}'>Select this URI</a>""".format(popup, url_r)
                },
                "id": url_r
                }
            return r
        else:
            return ''


class NetJsonEdgeSerializer(serializers.BaseSerializer):

    def to_representation(self, obj):
        ent_obj = obj.__class__.__name__
        ob_match = re.match(r'([A-Z][a-z]+)([A-Z][a-z]+)$', ent_obj)
        rel_a = 'related_' + ob_match.group(1).lower()
        rel_b = 'related_' + ob_match.group(2).lower()
        if rel_a == rel_b:
            rel_a += 'A'
            rel_b += 'B'
        r = {
            'source': getattr(obj, rel_a).pk,
            'target': getattr(obj, rel_b).pk,
            'id': obj.pk,
            'type': 'arrow',
            'data': dict()
        }
        r['data']['start_date'] = obj.start_date_written
        r['data']['end_date'] = obj.end_date_written
        r['data']['relation_type'] = obj.relation_type.name

        return r


class NetJsonNodeSerializer(serializers.BaseSerializer):

    def to_representation(self, obj):
        ent_obj = obj.__class__.__name__
        ent_url = reverse_lazy(
            'apis:apis_entities:generic_entities_edit_view',
            kwargs={
                'pk': str(obj.pk), 'entity': ent_obj.lower()
                }
            )
        tt = """<div class='arrow'></div>
            <div class='sigma-tooltip-header'>{}</div>
            <div class='sigma-tooltip-body'>
            <table>
                <tr><th>Type</th> <td>{}</td></tr>
                <tr><th>Entity</th> <td><a href='{}'>Link</a></td></tr>
            </table>
            <button class='small-button' onclick='expand_node("{}", {})'>expand</button>
            </div>""".format(str(obj), ent_obj, ent_url, ent_obj, obj.pk)
        r = {
            'type': ent_obj.lower(),
            'label': str(obj),
            'id': obj.pk,
            'tooltip': tt,
            'data': dict()}
        r['data']['uri'] = [x.uri for x in obj.uri_set.all()]
        r['data']['collections'] = [x.name for x in obj.collection.all()]
        r['data']['notes'] = obj.notes
        r['data']['references'] = obj.references
        r['data']['start_date'] = obj.start_date_written
        r['data']['end_date'] = obj.end_date_written
        if ent_obj.lower() != 'person':
            if obj.kind:
                r['data']['kind'] = obj.kind.name
        if ent_obj.lower() == 'place':
            r['data']['lat'] = obj.lat
            r['data']['lon'] = obj.lng
        if ent_obj.lower() == 'person':
            r['data']['profession'] = [x.name for x in obj.profession.all()]
            if obj.gender:
                r['data']['gender'] = obj.gender
        return r



from apis_highlighter.models import Annotation
from apis_core.apis_metainfo.models import Text
import re
from django.contrib.contenttypes.models import ContentType


def highlight_text(*args, **kwargs):
    ann_proj_pk = kwargs.pop('set_ann_proj', False)
    types = kwargs.pop('types', False)
    users_show = kwargs.pop('users_show', False)
    t_start = 0
    t_end = 0
    obj = args[-1]
    if isinstance(obj, str):
        obj = Text.objects.get(pk=obj)
    if not types or not users_show:
        return obj.text
    lst_annot = []
    queries = dict()
    if users_show:
        queries['user_added_id__in'] = users_show
    if ann_proj_pk:
        queries['annotation_project__pk'] = ann_proj_pk
    queries['text'] = obj
    for an in Annotation.objects.filter(**queries).order_by('start'):
        if types:
            test = False
            for m in an.entity_link.all():
                t = ContentType.objects.get_for_model(m)
                if str(t.pk) in types:
                    test = True
            if not test:
                continue
        if an.start >= t_start and an.start <= t_end:
            lst_annot[-1].append(an)
        else:
            lst_annot.append([an, ])
        t_start = an.start
        t_end = an.end
    if len(lst_annot) == 0:
        return obj.text
    html_return = obj.text[:lst_annot[0][0].start]
    end = ''
    lst_end = None
    for an in lst_annot:
        start = min([x.start for x in an])
        end = max([x.end for x in an])
        if len(an) > 1:
            start_span = '''<mark class="hl_text_complex" data-hl-type="complex" data-hl-start="{}"
            data-hl-end="{}" data-hl-text-id="{}">'''.format(start, end, obj.pk)
        else:
            start_span = an[0].get_html_markup()
        if lst_end:
            html_return += obj.text[lst_end:start] + start_span + obj.text[start:end] + '</mark>'
        else:
            html_return += start_span + obj.text[start:end] + '</mark>'
        lst_end = end
    html_return += obj.text[end:]
    if obj.text[0] == '\n':
        html_return = '-'+html_return[1:]
    return html_return


def highlight_textTEI(*args, **kwargs):
    user_pk = kwargs.pop('user', False)
    ann_proj_pk = kwargs.pop('ann_proj', False)
    obj = args[-1]
    t_start = 0
    t_end = 0
    if isinstance(obj, str):
        obj = Text.objects.get(pk=obj)
    lst_annot = []
    for an in Annotation.objects.filter(text=obj).order_by('start'):
        if an.start >= t_start and an.start <= t_end:
            lst_annot[-1].append(an)
        else:
            lst_annot.append([an, ])
        t_start = an.start
        t_end = an.end
    #print(lst_annot)
    if len(lst_annot) == 0:
        return obj.text
    html_return = obj.text[:lst_annot[0][0].start]
    end = ''
    lst_end = None
    for an in lst_annot:
        start = min([x.start for x in an])
        end = max([x.end for x in an])
        try:
            lst_classes = ' '.join([str(x.relation_type.pk) for x in an[0].entity_link.all()])
        except:
            try:
                lst_classes = ' '.join([str(x.kind.pk) for x in an[0].entity_link.all()])
            except:
                lst_classes = ''
        if len(an) > 1:
            start_span = '<name type="complex" hl-start="{}" hl-end="{}" hl-text-id="{}">'.format(start, end, obj.pk)
        else:
            try:
                entity_type = type(an[0].entity_link.all()[0]).__name__
                entity_pk = an[0].entity_link.all()[0].pk
                ent_lst_pk = []
                for x in dir(an[0].entity_link.all()[0]):
                    c = re.match('related_\w+_id', x)
                    if c:
                        ent_lst_pk.append(str(getattr(an[0].entity_link.all()[0], c.group(0))))
                if len(ent_lst_pk) == 0:
                    ent_lst_pk.append(str(an[0].entity_link.all()[0].pk))
            except:
                entity_type = ''
                entity_pk = ''
                ent_lst_pk = []
            try:
                entity_uri = an[0].entity_link.all()[0].uri_set.values_list('uri', flat=True)[0]
            except:
                entity_uri = 'internal db id: {}'.format(an[0].entity_link.all()[0].pk)
            start_span = '<name hl-type="simple" hl-start="{}" hl-end="{}" hl-text-id="{}" hl-ann-id="{}" type="{}" entity-pk="{}" related-entity-pk="{}">'.format(start, end, obj.pk, an[0].pk, entity_type, entity_pk, ','.join(ent_lst_pk))
            #'<span class="highlight hl_text_{}" data-hl-type="simple" data-hl-start="{}" data-hl-end="{}" data-hl-text-id="{}" data-hl-ann-id="{}" data-entity-class="{}" data-entity-pk="{}" data-related-entity-pk="{}">'.format(start, end, obj.pk, an[0].pk, entity_type, entity_pk, ','.join(ent_lst_pk))
        if lst_end:
            if len(an) > 1:
                html_return += obj.text[lst_end:start] + start_span + obj.text[start:end] + '</name>'
            else:
                html_return += obj.text[lst_end:start] + start_span + obj.text[start:end] + '<index><term>'+entity_uri+ '</term></index></name>'
        else:
            html_return += start_span + obj.text[start:end] + '</name>'
        lst_end = end
    html_return += obj.text[end:]
    return html_return



from django.conf.urls import url, include
from django.conf import settings
from django.contrib import admin
from rest_framework import routers
from apis_core.apis_entities.api_views import (
    InstitutionViewSet, PersonViewSet, PlaceViewSet, EventViewSet, WorkViewSet, PlaceGeoJsonViewSet,
    NetJsonViewSet
)
from django.views.static import serve
from django.contrib.auth.decorators import login_required
#from rest_framework_swagger.views import get_swagger_view
#from entities.views2 import GenericEntitiesCreateStanbolView
from apis_core.apis_metainfo.api_views import (
    CollectionViewSet, TextViewSet, SourceSerializerViewSet,
    UriSerializerViewSet, TempEntityClassViewSet)
from apis_core.apis_relations.api_views import (
    InstitutionInstitutionViewSet, InstitutionPlaceViewSet, InstitutionEventViewSet, InstitutionWorkViewSet,
    PersonInstitutionViewSet, PersonPlaceViewSet, PersonPersonViewSet, PersonEventViewSet, PersonWorkViewSet,
    PlaceWorkViewSet, PlaceEventViewSet, EventWorkViewSet, EventEventViewSet, WorkWorkViewSet,
    PlacePlaceViewSet)
from apis_core.apis_vocabularies.api_views import (
    UserViewSet, VocabNamesViewSet, TextTypeViewSet, CollectionTypeViewSet, VocabsBaseClassViewSet,
    InstitutionTypeViewSet, ProfessionTypeViewSet, PlaceTypeViewSet, EventTypeViewSet, WorkTypeViewSet,
    PersonInstitutionRelationViewSet, PersonPlaceRelationViewSet, PersonEventRelationViewSet, PersonWorkRelationViewSet,
    PersonPersonRelationViewSet, InstitutionInstitutionRelationViewSet, InstitutionPlaceRelationViewSet,
    InstitutionEventRelationViewSet, InstitutionWorkRelationViewSet, PlaceEventRelationViewSet,
    PlaceWorkRelationViewSet, EventWorkRelationViewSet, EventEventRelationViewSet, WorkWorkRelationViewSet,
    PlacePlaceRelationViewSet)

app_name = 'apis_core'

router = routers.DefaultRouter()
router.register(r'tempentity', TempEntityClassViewSet)
router.register(r'institution', InstitutionViewSet)
router.register(r'person', PersonViewSet)
router.register(r'place', PlaceViewSet)
router.register(r'event', EventViewSet)
router.register(r'work', WorkViewSet)
router.register(r'collection', CollectionViewSet)
router.register(r'text', TextViewSet, 'text')
router.register(r'source', SourceSerializerViewSet)
router.register(r'uri', UriSerializerViewSet)
router.register(r'institutioninstitution', InstitutionInstitutionViewSet)
router.register(r'institutioninstitutionrelation', InstitutionInstitutionRelationViewSet)
router.register(r'institutionplace', InstitutionPlaceViewSet)
router.register(r'institutionplacerelation', InstitutionPlaceRelationViewSet)
router.register(r'institutionevent', InstitutionEventViewSet)
router.register(r'institutioneventrelation', InstitutionEventRelationViewSet)
router.register(r'institutionwork', InstitutionWorkViewSet)
router.register(r'institutionworkrelation', InstitutionWorkRelationViewSet)
router.register(r'personinstitution', PersonInstitutionViewSet)
router.register(r'personinstitutionrelation', PersonInstitutionRelationViewSet)
router.register(r'personplace', PersonPlaceViewSet)
router.register(r'personplacerelation', PersonPlaceRelationViewSet)
router.register(r'personevent', PersonEventViewSet)
router.register(r'personeventrelation', PersonEventRelationViewSet)
router.register(r'personwork', PersonWorkViewSet)
router.register(r'personworkrelation', PersonWorkRelationViewSet)
router.register(r'personperson', PersonPersonViewSet)
router.register(r'personpersonrelation', PersonPersonRelationViewSet)
router.register(r'placework', PlaceWorkViewSet)
router.register(r'placeworkrelation', PlaceWorkRelationViewSet)
router.register(r'placeevent', PlaceEventViewSet)
router.register(r'placeeventrelation', PlaceEventRelationViewSet)
router.register(r'placeplace', PlacePlaceViewSet)
router.register(r'placeplacerelation', PlacePlaceRelationViewSet)
router.register(r'eventwork', EventWorkViewSet)
router.register(r'eventworkrelation', EventWorkRelationViewSet)
router.register(r'eventevent', EventEventViewSet)
router.register(r'eventeventrelation', EventEventRelationViewSet)
router.register(r'workwork', WorkWorkViewSet)
router.register(r'workworkrelation', WorkWorkRelationViewSet)
router.register(r'texttype', TextTypeViewSet)
router.register(r'collectiontype', CollectionTypeViewSet)
router.register(r'vocabsbaseclass', VocabsBaseClassViewSet)
router.register(r'institutiontype', InstitutionTypeViewSet)
router.register(r'professiontype', ProfessionTypeViewSet)
router.register(r'placetype', PlaceTypeViewSet)
router.register(r'eventtype', EventTypeViewSet)
router.register(r'worktype', WorkTypeViewSet)
router.register(r'users', UserViewSet)
router.register(r'GeoJsonPlace', PlaceGeoJsonViewSet, 'PlaceGeoJson')
router.register(r'NetJson', NetJsonViewSet, 'NetJson')
router.register(r'VocabNames', VocabNamesViewSet)

if 'apis_highlighter' in settings.INSTALLED_APPS:
    from apis_highlighter.api_views import (
        HighlighterProjectViewSet, HighlighterTextHighViewSet, HighlighterMenuEntryViewSet,
        HighlighterHighlightTextViewSet, HighlighterVocabularyAPIViewSet, HighlighterAnnotationViewSet
    )
    router.register(r'HLProjects', HighlighterProjectViewSet)
    router.register(r'HLTextHigh', HighlighterTextHighViewSet)
    router.register(r'HLMenuEntry', HighlighterMenuEntryViewSet)
    router.register(r'HLTextHighlighter', HighlighterHighlightTextViewSet, 'HLTextHighlighter')
    router.register(r'HLVocabularyAPI', HighlighterVocabularyAPIViewSet)
    router.register(r'HLAnnotation', HighlighterAnnotationViewSet)

urlpatterns = [
    url(r'^admin/', admin.site.urls),
    url(r'labels/', include('apis_core.apis_labels.urls', namespace='apis_labels')),
    url(r'tei/', include('apis_core.apis_tei.tei_urls', namespace='apis_tei')),
    url(r'entities/', include('apis_core.apis_entities.urls', namespace='apis_entities')),
    url(r'visuals/', include('apis_core.apis_vis.urls', namespace='apis_visuals')),
    url(r'relations/', include('apis_core.apis_relations.urls', namespace='apis_relations')),
    url(r'vocabularies/', include('apis_core.apis_vocabularies.urls', namespace='apis_vocabularies')),
    #url(r'^autocomplete/', include('autocomplete_light.urls')),
    url(r'^api/', include(router.urls)),    #routers do not support namespaces out of the box
    url(r'^api2/', include('apis_core.apis_entities.api_urls', namespace="apis_api2")),
    url(r'^api-auth/', include('rest_framework.urls', namespace='rest_framework')),
    #url(r'^api-schema/', schema_view),
    url(r'^apis-vis/', include('apis_core.apis_vis.urls', namespace='apis_vis')),
    url(r'^docs/(?P<path>.*)', login_required(serve), {'document_root': 'apis-core/docs/_build/html'}, 'docs'),
    #url(r'^docs/', include('sphinxdoc.urls')),
    #url(r'^accounts/', include('registration.backends.simple.urls')),
]

if 'apis_highlighter' in settings.INSTALLED_APPS:
    urlpatterns.append(url(r'highlighter/', include('apis_highlighter.urls', namespace='highlighter')))

if 'apis_fulltext_download' in settings.INSTALLED_APPS:
    urlpatterns.append(url(r'fulltext_download/', include('apis_fulltext_download.urls', namespace='apis_fulltext_download')))

if settings.DEBUG:
    if 'debug_toolbar' in settings.INSTALLED_APPS:
        import debug_toolbar
        urlpatterns = [
            url(r'^__debug__/', include(debug_toolbar.urls)),
        ] + urlpatterns



from django.db import models
from django.conf import settings
# from reversion import revisions as reversion
import reversion
from django.db.models.signals import post_save, m2m_changed
from django.dispatch import receiver
from django.urls import reverse
from guardian.shortcuts import assign_perm, remove_perm
from django.conf import settings
from django.contrib.auth.models import Group

from apis_core.apis_metainfo.models import TempEntityClass, Uri, Text, Collection
from apis_core.apis_labels.models import Label
from apis_core.apis_vocabularies.models import (
    ProfessionType, PlaceType, InstitutionType,
    EventType, Title, WorkType)

import re
import unicodedata


BASE_URI = getattr(settings, "APIS_BASE_URI", 'http://apis.info/')


@reversion.register(follow=['tempentityclass_ptr'])
class Person(TempEntityClass):
    """ A temporalized entity to model a human beeing."""

    GENDER_CHOICES = (('female', 'female'), ('male', 'male'))
    first_name = models.CharField(
        max_length=255,
        help_text='The persons´s forename. In case of more then one name...',
        blank=True,
        null=True)
    profession = models.ManyToManyField(ProfessionType, blank=True)
    title = models.ManyToManyField(Title, blank=True)
    gender = models.CharField(
        max_length=15, choices=GENDER_CHOICES,
        blank=True)

    def __str__(self):
        if self.first_name != "" and self.name != "":
            return "{}, {}".format(self.name, self.first_name)
        elif self.first_name != "" and self.name == "":
            return "{}, {}".format("no surename provided", self.first_name)
        elif self.first_name == "" and self.name != "":
            return self.name
        elif self.first_name == "" and self.name == "":
            return "no name provided"

    def get_or_create_uri(uri):
        try:
            if re.match(r'^[0-9]*$', uri):
                p = Person.objects.get(pk=uri)
            else:
                p = Person.objects.get(uri__uri=uri)
            return p
        except:
            return False

    def save(self, *args, **kwargs):
        if self.first_name:
            # secure correct unicode encoding
            if self.first_name != unicodedata.normalize('NFC', self.first_name):
                self.first_name = unicodedata.normalize('NFC', self.first_name)
        super(Person, self).save(*args, **kwargs)
        return self


@reversion.register(follow=['tempentityclass_ptr'])
class Place(TempEntityClass):
    """ A temporalized entity to model a place"""

    kind = models.ForeignKey(PlaceType, blank=True, null=True,
                             on_delete=models.SET_NULL)
    lat = models.FloatField(blank=True, null=True, verbose_name='latitude')
    lng = models.FloatField(blank=True, null=True, verbose_name='longitude')

    def __str__(self):
        if self.name != "":
            return self.name
        else:
            return "no name provided"

    def get_or_create_uri(uri):
        try:
            if re.match(r'^[0-9]*$', uri):
                p = Place.objects.get(pk=uri)
            else:
                p = Place.objects.get(uri__uri=uri)
            return p
        except:
            return False


@reversion.register(follow=['tempentityclass_ptr'])
class Institution(TempEntityClass):
    kind = models.ForeignKey(InstitutionType, blank=True, null=True,
                             on_delete=models.SET_NULL)
    homepage = models.URLField(blank=True, null=True)

    def __str__(self):
        if self.name != "":
            return self.name
        else:
            return "no name provided"

    def get_or_create_uri(uri):
        try:
            if re.match(r'^[0-9]*$', uri):
                p = Institution.objects.get(pk=uri)
            else:
                p = Institution.objects.get(uri__uri=uri)
                print(p)
            return p
        except:
            print('returned false')
            return False


@reversion.register(follow=['tempentityclass_ptr'])
class Event(TempEntityClass):
    kind = models.ForeignKey(EventType, blank=True, null=True,
                             on_delete=models.SET_NULL)

    def __str__(self):
        if self.name != "":
            return self.name
        else:
            return "no name provided"

    def get_or_create_uri(uri):
        try:
            if re.match(r'^[0-9]*$', uri):
                p = Event.objects.get(pk=uri)
            else:
                p = Event.objects.get(uri__uri=uri)
            return p
        except:
            return False


@reversion.register(follow=['tempentityclass_ptr'])
class Work(TempEntityClass):
    kind = models.ForeignKey(WorkType, blank=True, null=True,
                             on_delete=models.SET_NULL)

    def __str__(self):
        if self.name != "":
            return self.name
        else:
            return "no name provided"

    def get_or_create_uri(uri):
        try:
            if re.match(r'^[0-9]*$', uri):
                p = Work.objects.get(pk=uri)
            else:
                p = Work.objects.get(uri__uri=uri)
            return p
        except:
            return False


@receiver(post_save, sender=Event, dispatch_uid="create_default_uri")
@receiver(post_save, sender=Work, dispatch_uid="create_default_uri")
@receiver(post_save, sender=Institution, dispatch_uid="create_default_uri")
@receiver(post_save, sender=Person, dispatch_uid="create_default_uri")
@receiver(post_save, sender=Place, dispatch_uid="create_default_uri")
def create_default_uri(sender, instance, **kwargs):
    uri = Uri.objects.filter(entity=instance)
    if uri.count() == 0:
        uri_c = "http://{}{}".format(BASE_URI, reverse('apis_core:apis_api2:GetEntityGeneric', kwargs={'pk': instance.pk}))
        uri2 = Uri(
            uri=uri_c,
                domain='apis default',
                entity=instance)
        uri2.save()


@receiver(m2m_changed, sender=Event.collection.through, dispatch_uid="create_object_permissions")
@receiver(m2m_changed, sender=Work.collection.through, dispatch_uid="create_object_permissions")
@receiver(m2m_changed, sender=Institution.collection.through, dispatch_uid="create_object_permissions")
@receiver(m2m_changed, sender=Person.collection.through, dispatch_uid="create_object_permissions")
@receiver(m2m_changed, sender=Place.collection.through, dispatch_uid="create_object_permissions")
def create_object_permissions(sender, instance, **kwargs):
    if kwargs['action'] == 'pre_add':
        perms = []
        for j in kwargs['model'].objects.filter(pk__in=kwargs['pk_set']):
            perms.extend(j.groups_allowed.all())
        for x in perms:
            assign_perm('change_'+instance.__class__.__name__.lower(), x, instance)
            assign_perm('delete_'+instance.__class__.__name__.lower(), x, instance)
    elif kwargs['action'] == 'post_remove':
        perms = []
        perms_keep = []
        for j in kwargs['model'].objects.filter(pk__in=kwargs['pk_set']):
            perms.extend(j.groups_allowed.all())
        for u in instance.collection.all():
            perms_keep.extend(u.groups_allowed.all())
        rm_perms = set(perms) - set(perms_keep)
        for x in rm_perms:
            remove_perm('change_'+instance.__class__.__name__.lower(), x, instance)
            remove_perm('delete_'+instance.__class__.__name__.lower(), x, instance)


@receiver(m2m_changed, sender=Collection.groups_allowed.through, dispatch_uid="add_usergroup_collection")
def add_usergroup_collection(sender, instance, **kwargs):
    if kwargs['action'] == 'pre_add':
        for x in kwargs['model'].objects.filter(pk__in=kwargs['pk_set']):
            for z in ['change', 'delete']:
                for y in [Person, Institution, Place, Event, Work]:
                    assign_perm(
                        z+'_'+y.__name__.lower(),
                        x,
                        y.objects.filter(collection=instance))


if 'registration' in getattr(settings, 'INSTALLED_APPS', []):
    from registration.backends.simple.views import RegistrationView
    from registration.signals import user_registered

    @receiver(user_registered, sender=RegistrationView, dispatch_uid="add_registered_user_to_group")
    def add_user_to_group(sender, user, request, **kwargs):
        user_group = getattr(settings, "APIS_AUTO_USERGROUP", None)
        if user_group is not None:
            user.groups.add(Group.objects.get(name=user_group))



#!/usr/bin/python
# -*- coding: utf-8 -*-
from .models import Place, Person, Institution, Event, Work
from apis_core.apis_metainfo.models import Uri, Collection
from apis_core.default_settings.NER_settings import autocomp_settings as ac_settings
from django.conf import settings
from django.core.exceptions import FieldError
from .custom_autocompletes import *

from dal import autocomplete
from django.db.models import Q
from django import http
from django.contrib.contenttypes.models import ContentType

import requests
import json
import operator
from functools import reduce
import dateutil.parser
import re

from apis_core.apis_vocabularies.models import VocabsBaseClass


class CustomEntityAutocompletes(object):
    """A class for collecting all the custom autocomplete functions for one entity.

    Attributes:

    - self.entity: (string) entity types
    - self.more: (boolean) if more results can be fetched (pagination)
    - self.page_size: (integer) page size
    - self.results: (list) results
    - self.query: (string) query string

    Methods:
    - self.more(): fetch more results
    """

    def __init__(self, entity, query, page_size=20, offset=0, *args, **kwargs):
        """
        :param entity: (string) entity type to fetch additional autocompletes for
        """
        func_list = {}
        if entity not in func_list.keys():
            self.results = None
            return None
        res = []
        more = dict()
        more_gen = False
        for x in func_list[entity]:
            res2 = x().query(query, page_size, offset)
            if len(res2) == page_size:
                more[x.__name__] = (True, offset+1)
                more_gen = True
            res.extend(res2)
        self.results = res
        self.page_size = page_size
        self.more = more_gen
        self._more_dict = more
        self.query = query
        self.offset = offset

    def get_more(self):
        """
        Function to retrieve more results.
        """
        res4 = []
        for key, value in self._more_dict.items():
            if value[0]:
                res3 = globals()[key](self.query, self.page_size, value[1])
                if len(res3) == self.page_size:
                    self._more_dict[key] = (True, value[1]+1)
                else:
                    self._more_dict[key] = (False, value[1])
                self.results.extend(res3)
                res4.extend(res3)
        return res4


class GenericEntitiesAutocomplete(autocomplete.Select2ListView):

    @staticmethod
    def parse_stanbol_object(obj, key, *args):
        if len(args) > 0:
            lst1 = args[0]
        else:
            lst1 = None
        if obj[1] == 'GNDDate':
            if lst1 is not None:
                try:
                    return dateutil.parser.parse(lst1[key][0]['value'])
                except:
                    return lst1[key][0]['value']
            else:
                return obj[0]
        elif obj[1] == 'String':
            if lst1 is not None:
                return lst1[key][0]['value']
            else:
                return obj[0]
        elif obj[1] == 'gndLong':
            if lst1 is not None:
                try:
                    return re.search('Point \( [+-]([0-9\.]+) [+-]([0-9\.]+)', lst1[key][0]['value']).group(1)
                except:
                    print('extract fails')
                    return None
            else:
                print('no match')

    def get(self, request, *args, **kwargs):
        page_size = 20
        offset = (int(self.request.GET.get('page', 1))-1)*page_size
        ac_type = self.kwargs['entity']
        db_include = self.kwargs.get('db_include', False)
        choices = []
        headers = {'Content-Type': 'application/json'}
        ent_model = ContentType.objects.get(
            app_label='apis_entities', model=ac_type.lower()
        ).model_class()
        if self.q.startswith('http'):
            res = ent_model.objects.filter(uri__uri=self.q.strip())
        else:
            q1 = re.match('^([^\[]+)\[([^\]]+)\]$', self.q)
            if q1:
                q = q1.group(1).strip()
                q3 = q1.group(2).split(',')
                q3 = [e.strip() for e in q3]
            else:
                q = re.match('^[^[]+', self.q).group(0)
                q3 = False
            if re.match('^[^*]+\*$', q.strip()):
                search_type = '__istartswith'
                q = re.match('^([^*]+)\*$', q.strip()).group(1)
            elif re.match('^\*[^*]+$', q.strip()):
                search_type = '__iendswith'
                q = re.match('^\*([^*]+)$', q.strip()).group(1)
            elif re.match('^\*[^*]+\*$', q.strip()):
                search_type = '__icontains'
                q = re.match('^\*([^*]+)\*$', q.strip()).group(1)
            elif re.match('^[^*]+$', q.strip()):
                search_type = ''
                q = q.strip()

            arg_list = [Q(**{x+search_type: q}) for x in settings.APIS_ENTITIES[ac_type.title()]['search']]
            res = ent_model.objects.filter(reduce(operator.or_, arg_list)).distinct()
            if q3:
                f_dict2 = {}
                for fd in q3:
                    f_dict2[fd.split('=')[0].strip()] = fd.split('=')[1].strip()
                try:
                    res = res.filter(**f_dict2)
                except Exception as e:
                    choices.append({'name': str(e)})
        test_db = True
        test_stanbol = False
        test_stanbol_list = dict()
        more = True
        if not db_include:
            for r in res[offset:offset+page_size]:
                f = dict()
                dataclass = ""
                try:
                    f['id'] = Uri.objects.filter(entity=r)[0].uri
                except:
                    continue
                if hasattr(r, 'lng'):
                    if r.lng and r.lat:
                        dataclass = 'data-vis-tooltip="{}" data-lat="{}" \
                        data-long="{}"  class="apis-autocomplete-span"'.format(ac_type, r.lat, r.lng)
                f['text'] = '<span {}><small>db</small> {}</span>'.format(dataclass, str(r))
                choices.append(f)
            if len(choices) < page_size:
                test_db = False
        else:
            test_db = False
        if ac_type.title() in ac_settings.keys():
            for y in ac_settings[ac_type.title()]:
                ldpath = ""
                for d in y['fields'].keys():
                    ldpath += "{} = <{}>;\n".format(d, y['fields'][d][0])
                if self.q.startswith('http'):
                    q = False
                    match_url_geo = re.search(r'geonames[^0-9]+([0-9]+)', self.q.strip())
                    if match_url_geo:
                        url = 'http://sws.geonames.org/{}/'.format(match_url_geo.group(1))
                    else:
                        url = self.q.strip()
                    params = {'id': url, 'ldpath': ldpath}
                    headers = {'Content-Type': 'application/json'}
                    w = requests.get(y['url'].replace('find', 'entity'), params=params, headers=headers)
                    res3 = dict()
                    ldpath_fields = [y['fields'][d][0] for d in y['fields'].keys()]
                    print(w.status_code)
                    if w.status_code == 200:
                        for x in w.json()['representation'].keys():
                            if x in ldpath_fields:
                                for d in y['fields'].keys():
                                    if y['fields'][d][0] == x:
                                        res3[d] = w.json()['representation'][x]
                        res = dict()
                        res3['id'] = w.json()['id']
                        res['results'] = [res3]
                    else:
                        continue
                else:

                    data = {
                        'limit': page_size,
                        'ldpath': ldpath,
                        'offset': offset,
                        'constraints': [{
                            "type": "text",
                            "patternType": "wildcard",
                            "field": "http://www.w3.org/2000/01/rdf-schema#label",
                            "text": q.split()
                        }]
                    }
                    if q3 and 'search fields' in y.keys():
                        for fd in q3:
                            fd = [fd2.strip() for fd2 in fd.split('=')]
                            if fd[0] in y['search fields'].keys():
                                fd3 = y['search fields'][fd[0]]
                                v = False
                                if isinstance(fd3[1], dict):
                                    if fd[1] in fd3[1].keys():
                                        v = fd3[1][fd[1]]
                                else:
                                    v = fd3[1](fd[1])
                                if fd3[2] == 'reference' and v:
                                    fd_4 = {
                                        'type': 'reference',
                                        'value': v,
                                        'field': fd3[0]
                                    }
                                    data['constraints'].append(fd_4)
                                elif fd3[2] == 'date_exact' and v:
                                    fd_4 = {
                                        'type': 'value',
                                        'value': v,
                                        'field': fd3[0],
                                        "datatype": "xsd:dateTime"
                                    }
                                    data['constraints'].append(fd_4)
                                elif fd3[2] == 'date_gt' and v:
                                    fd_4 = {
                                        'type': 'range',
                                        'lowerBound': v,
                                        'upperBound': "2100-12-31T23:59:59.999Z",
                                        'field': fd3[0],
                                        "datatype": "xsd:dateTime"
                                    }
                                    data['constraints'].append(fd_4)
                                elif fd3[2] == 'date_lt' and v:
                                    fd_4 = {
                                        'type': 'range',
                                        'lowerBound': "1-01-01T23:59:59.999Z",
                                        'upperBound': v,
                                        'field': fd3[0],
                                        "datatype": "xsd:dateTime"
                                    }
                                    data['constraints'].append(fd_4)
                            else:
                                choices.append({'name': 'No additional query setting for Stanbol'})
                    try:
                        url2 = y['url'].replace('find', 'query')
                        r = requests.post(url2, data=json.dumps(data), headers=headers)
                        if r.status_code != 200:
                            choices.append({'name': 'Connection to Stanbol failed'})
                            continue
                        res = r.json()
                    except Exception as e:
                        choices.append({'name': 'Connection to Stanbol failed'})
                        print(e)
                        continue
                if len(res['results']) < page_size:
                    test_stanbol_list[y['url']] = False
                else:
                    test_stanbol_list[y['url']] = True
                for x in res['results']:
                    f = dict()
                    dataclass = ""
                    name = x['name'][0]['value']
                    if ac_settings['score'] in x.keys():
                        score = str(x[ac_settings['score']][0]['value'])
                    else:
                        score = 'NA'
                    id = x[ac_settings['uri']]
                    score = score
                    f['id'] = id
                    source = y['source']
                    if 'lat' in x.keys() and 'long' in x.keys():
                        dataclass = 'data-vis-tooltip="{}" \
                        data-lat="{}" data-long="{}"'.format(
                            ac_type, x['lat'][0]['value'], x['long'][0]['value'])
                    if 'descr' in x.keys():
                        descr = x['descr'][0]['value']
                    else:
                        descr = None
                    f['text'] = '<span {} class="apis-autocomplete-span"><small>{}</small> <b>{}</b>\
                    ({}): {}</span>'.format(dataclass, source, name, score, descr)
                    choices.append(f)
            for k in test_stanbol_list.keys():
                if test_stanbol_list[k]:
                    test_stanbol = True
        else:
            test_stanbol = False
        cust_auto_more = False
        if q:
            cust_auto = CustomEntityAutocompletes(ac_type, q, page_size=page_size, offset=offset)
            if cust_auto.results is not None:
                cust_auto_more = cust_auto.more
                if len(cust_auto.results) > 0:
                    choices.extend(cust_auto.results)
        if not test_db and not test_stanbol and not cust_auto_more:
            more = False
        return http.HttpResponse(json.dumps({
            'results': choices + [],
            'pagination': {'more': more}
        }), content_type='application/json')


class GenericVocabulariesAutocomplete(autocomplete.Select2ListView):
    def get(self, request, *args, **kwargs):
        vocab = self.kwargs['vocab']
        direct = self.kwargs['direct']
        q = self.q
        vocab_model = ContentType.objects.get(app_label='apis_vocabularies', model=vocab).model_class()
        if direct == 'normal':
            if vocab_model.__bases__[0] == VocabsBaseClass:
                choices = [{'id': x.pk, 'text': x.name} for x in vocab_model.objects.filter(name__icontains=q)]
            else:
                choices = [{'id': x.pk, 'text': x.label} for x in vocab_model.objects.filter(
                    Q(name__icontains=q) | Q(name_reverse__icontains=q))]
        elif direct == 'reverse':
            choices = [{'id': x.pk, 'text': x.label_reverse} for x in vocab_model.objects.filter(
                Q(name__icontains=q) | Q(name_reverse__icontains=q))]
        return http.HttpResponse(json.dumps({
            'results': choices + []
        }), content_type='application/json')


class GenericNetworkEntitiesAutocomplete(autocomplete.Select2ListView):
    def get(self, request, *args, **kwargs):
        entity = self.kwargs['entity']
        q = self.q
        if q.startswith('cl:'):
            res = Collection.objects.filter(name__icontains=q[3:])
            results = [{'id': 'cl:'+str(x.pk), 'text': x.name} for x in res]
        else:
            ent_model = ContentType.objects.get(
                app_label__startswith='apis_', model=entity
            ).model_class()
            try:
                arg_list = [
                    Q(
                        **{x + '__icontains': q}
                    ) for x in settings.APIS_ENTITIES[entity.title()]['search']
                ]
            except KeyError:
                arg_list = [
                    Q(
                        **{x + '__icontains': q}
                    ) for x in ['name']
                ]
            try:
                res = ent_model.objects.filter(reduce(operator.or_, arg_list)).distinct()
            except FieldError:
                arg_list = [
                    Q(
                        **{x + '__icontains': q}
                    ) for x in ['text']
                ]
                res = ent_model.objects.filter(reduce(operator.or_, arg_list)).distinct()
            results = [{'id': x.pk, 'text': str(x)} for x in res]
        return http.HttpResponse(json.dumps({
            'results': results
        }), content_type='application/json')



from rest_framework import serializers
from django.contrib.auth.models import User

from .models import (
    InstitutionInstitutionRelation, TextType, CollectionType, VocabsBaseClass,
    InstitutionType, ProfessionType, PlaceType, PersonInstitutionRelation, InstitutionPlaceRelation,
    PersonPlaceRelation, VocabNames, PersonPersonRelation, PersonEventRelation, PersonWorkRelation,
    InstitutionEventRelation, InstitutionWorkRelation, PlaceEventRelation, PlaceWorkRelation, PlacePlaceRelation,
    EventWorkRelation, EventEventRelation, WorkWorkRelation, EventType, WorkType, LabelType)


###########################################################
#
# Meta- Serializers
#
##########################################################


class UserAccSerializer(serializers.ModelSerializer):

    class Meta:
        model = User
        fields = ['username', 'email']


class CollectionTypeSerializer(serializers.HyperlinkedModelSerializer):

    class Meta:
        fields = '__all__'
        model = CollectionType


class VocabsBaseClassSerializer(serializers.HyperlinkedModelSerializer):

    class Meta:
        fields = '__all__'
        model = VocabsBaseClass


class VocabNamesSerializer(serializers.HyperlinkedModelSerializer):

    class Meta:
        fields = '__all__'
        model = VocabNames


############################################################
#
# Entity Types
#
###########################################################


class TextTypeSerializer(serializers.HyperlinkedModelSerializer):

    class Meta:
        fields = '__all__'
        model = TextType


class InstitutionTypeSerializer(serializers.HyperlinkedModelSerializer):

    class Meta:
        fields = '__all__'
        model = InstitutionType


class ProfessionTypeSerializer(serializers.HyperlinkedModelSerializer):

    class Meta:
        fields = '__all__'
        model = ProfessionType


class PlaceTypeSerializer(serializers.HyperlinkedModelSerializer):

    class Meta:
        fields = '__all__'
        model = PlaceType


class EventTypeSerializer(serializers.HyperlinkedModelSerializer):

    class Meta:
        fields = '__all__'
        model = EventType


class WorkTypeSerializer(serializers.HyperlinkedModelSerializer):

    class Meta:
        fields = '__all__'
        model = WorkType


class LabelTypeSerializer(serializers.ModelSerializer):

    class Meta:
        fields = ('id', 'name')
        model = LabelType

#####################################################################
#
# Relations
#
####################################################################


class InstitutionInstitutionRelationSerializer(serializers.HyperlinkedModelSerializer):

    class Meta:
        fields = '__all__'
        model = InstitutionInstitutionRelation


class PersonPersonRelationSerializer(serializers.HyperlinkedModelSerializer):

    class Meta:
        fields = '__all__'
        model = PersonPersonRelation


class PersonInstitutionRelationSerializer(serializers.HyperlinkedModelSerializer):

    class Meta:
        fields = '__all__'
        model = PersonInstitutionRelation


class PersonPlaceRelationSerializer(serializers.HyperlinkedModelSerializer):

    class Meta:
        fields = '__all__'
        model = PersonPlaceRelation


class PersonEventRelationSerializer(serializers.HyperlinkedModelSerializer):

    class Meta:
        fields = '__all__'
        model = PersonEventRelation


class PersonWorkRelationSerializer(serializers.HyperlinkedModelSerializer):

    class Meta:
        fields = '__all__'
        model = PersonWorkRelation


class InstitutionPlaceRelationSerializer(serializers.HyperlinkedModelSerializer):

    class Meta:
        fields = '__all__'
        model = InstitutionPlaceRelation


class InstitutionEventRelationSerializer(serializers.HyperlinkedModelSerializer):

    class Meta:
        fields = '__all__'
        model = InstitutionEventRelation


class InstitutionWorkRelationSerializer(serializers.HyperlinkedModelSerializer):

    class Meta:
        fields = '__all__'
        model = InstitutionWorkRelation


class PlaceEventRelationSerializer(serializers.HyperlinkedModelSerializer):

    class Meta:
        fields = '__all__'
        model = PlaceEventRelation


class PlaceWorkRelationSerializer(serializers.HyperlinkedModelSerializer):

    class Meta:
        fields = '__all__'
        model = PlaceWorkRelation


class PlacePlaceRelationSerializer(serializers.HyperlinkedModelSerializer):

    class Meta:
        fields = '__all__'
        model = PlacePlaceRelation


class EventWorkRelationSerializer(serializers.HyperlinkedModelSerializer):

    class Meta:
        fields = '__all__'
        model = EventWorkRelation


class EventEventRelationSerializer(serializers.HyperlinkedModelSerializer):

    class Meta:
        fields = '__all__'
        model = EventEventRelation


class WorkWorkRelationSerializer(serializers.HyperlinkedModelSerializer):

    class Meta:
        fields = '__all__'
        model = WorkWorkRelation



from django import forms
#from autocomplete_light import shortcuts as al
from crispy_forms.helper import FormHelper
from crispy_forms.layout import Submit
from crispy_forms.layout import Layout, Fieldset, ButtonHolder, Submit, Div

from .autocomplete_light_registry import LabelAutocomplete
from .models import Label


class LabelForm(forms.ModelForm):
    isoCode = forms.CharField(required=True,
                              #widget=al.TextWidget('LabelAutocomplete'),
                              label="ISO 639-3")
    label = forms.CharField(required=True, help_text="The entities label or name.")

    class Meta:
        model = Label
        fields = ['temp_entity', 'label', 'isoCode']

    def __init__(self, *args, **kwargs):
        super(LabelForm, self).__init__(*args, **kwargs)
        self.helper = FormHelper()
        self.helper.form_tag = False

    # self.helper.add_input(Submit('submit','Speichern'))



from rest_framework.test import APIClient
from django.test import TestCase
from rest_framework.authtoken.models import Token

from .models import Place, Person
from relations.models import PersonPlace
from vocabularies.models import PersonPlaceRelation


class PersonModelTestCase(TestCase):
    name = 'test name'
    first_name = 'test first name'
    place_name = 'test place name'
    start_date = '1.3.1930'
    end_date = '4.6.1960'

    @classmethod
    def setUpTestData(cls):
        # Set up data for the whole TestCase
        cls.person = Person.objects.create(name=cls.name, first_name=cls.first_name)
        cls.place = Place.objects.create(name=cls.place_name)
        token = Token.objects.get(user__username='lauren')
        client = APIClient()
        client.credentials(HTTP_AUTHORIZATION='Token ' + token.key)
        cls.c = client

    def test_apiget(self):
        token = Token.objects.get(user__username='lauren')
        client = APIClient()
        client.credentials(HTTP_AUTHORIZATION='Token ' + token.key)


from django.db import models
from reversion import revisions as reversion
from django.contrib.auth.models import User
from django.utils.functional import cached_property
import re

import unicodedata



@reversion.register()
class VocabNames(models.Model):
    """List of Vocabulary names to allow the easy retrieval of Vovcabulary names and classes from the VocabsBaseClass"""
    name = models.CharField(max_length=255)

    def get_vocab_label(self):
        return re.sub( r"([A-Z])", r" \1", self.name).strip()


@reversion.register()
class VocabsBaseClass(models.Model):
    """ An abstract base class for other classes which contain so called
    'controlled vocablury' to describe subtypes of main temporalized
    entites"""
    choices_status = (('rej', 'rejected'), ('ac', 'accepted'), ('can', 'candidate'), ('del', 'deleted'))
    name = models.CharField(max_length=255, verbose_name='Name')
    description = models.TextField(
        blank=True,
        help_text="Brief description of the used term.")
    parent_class = models.ForeignKey('self', blank=True, null=True,
                                     on_delete=models.CASCADE)
    status = models.CharField(max_length=4, choices=choices_status, default='can')
    userAdded = models.ForeignKey(User, blank=True, null=True,
                                  on_delete=models.SET_NULL)  #changed from default=12
    vocab_name = models.ForeignKey(VocabNames, blank=True, null=True,
                                   on_delete=models.SET_NULL)

    def __str__(self):
        return self.name

    def save(self, *args, **kwargs):
        d, created = VocabNames.objects.get_or_create(name=type(self).__name__)
        self.vocab_name = d
        if self.name != unicodedata.normalize('NFC', self.name):    #secure correct unicode encoding
            self.name = unicodedata.normalize('NFC', self.name)
        super(VocabsBaseClass, self).save(*args, **kwargs)
        return self

    @cached_property
    def label(self):
        d = self
        res = self.name
        while d.parent_class:
            res = d.parent_class.name + ' >> ' + res
            d = d.parent_class
        return res



@reversion.register(follow=['vocabsbaseclass_ptr'])
class RelationBaseClass(VocabsBaseClass):
    """ An abstract base class for other classes which contain so called
    'controlled vocablury' to describe the relations between main temporalized
    entites ('db_')"""

    name_reverse = models.CharField(
        max_length=255,
        verbose_name='Name reverse',
        help_text='Inverse relation like: "is sub-class of" vs. "is super-class of".',
        blank=True)

    def __str__(self):
        return self.name

    @cached_property
    def label_reverse(self):
        d = self
        if len(self.name_reverse) < 1:
            res = '(' + self.name + ')'
        else:
            res = self.name_reverse
        while d.parent_class:
            try:
                t = RelationBaseClass.objects.get(pk=d.parent_class.pk).name_reverse
                if len(t) < 1:
                    t = '(' + d.parent_class.name + ')'
            except:
                t = '(' + d.parent_class.name + ')'
            res = t + ' >> ' + res
            d = d.parent_class
        return res

    def save(self, *args, **kwargs):
        if self.name_reverse != unicodedata.normalize('NFC', self.name_reverse):    #secure correct unicode encoding
            self.name_reverse = unicodedata.normalize('NFC', self.name_reverse)
        super(RelationBaseClass, self).save(*args, **kwargs)
        return self


@reversion.register()
class VocabsUri(models.Model):
    """Class to store URIs for imported types. URI class from metainfo is not
    used in order to keep the vocabularies module/app seperated from the rest of the application.
    """
    uri = models.URLField()
    domain = models.CharField(max_length=255, blank=True)
    rdf_link = models.URLField(blank=True)
    vocab = models.ForeignKey(VocabsBaseClass, blank=True, null=True,
                              on_delete=models.CASCADE)
    # loaded: set to True when RDF was loaded and parsed into the data model
    loaded = models.BooleanField(default=False)
    # loaded_time: Timestamp when file was loaded and parsed
    loaded_time = models.DateTimeField(blank=True, null=True)

    def __str__(self):
        return self.uri

#######################################################################
#
#   entity types
#
#######################################################################


@reversion.register(follow=['vocabsbaseclass_ptr'])
class WorkType(VocabsBaseClass):
    """Holds controlled vocabularies about work-types"""
    pass


@reversion.register(follow=['vocabsbaseclass_ptr'])
class Title(VocabsBaseClass):
    """A person´s (academic) title"""
    abbreviation = models.CharField(max_length=10, blank=True)


@reversion.register(follow=['vocabsbaseclass_ptr'])
class ProfessionType(VocabsBaseClass):
    """Holds controlled vocabularies about profession-types"""
    pass


@reversion.register(follow=['vocabsbaseclass_ptr'])
class PlaceType(VocabsBaseClass):
    """Holds controlled vocabularies about place-types"""
    pass


@reversion.register(follow=['vocabsbaseclass_ptr'])
class InstitutionType(VocabsBaseClass):
    """Holds controlled vocabularies about institution-types"""
    pass


@reversion.register(follow=['vocabsbaseclass_ptr'])
class EventType(VocabsBaseClass):
    """Holds controlled vocabularies about event-types"""
    pass


@reversion.register(follow=['vocabsbaseclass_ptr'])
class LabelType(VocabsBaseClass):
    """Holds controlled vocabularies about label-types"""
    pass


@reversion.register(follow=['vocabsbaseclass_ptr'])
class CollectionType(VocabsBaseClass):
    """e.g. reseachCollection, importCollection """
    pass


@reversion.register(follow=['vocabsbaseclass_ptr'])
class TextType(VocabsBaseClass):
    """used to store the Text types for the forms"""
    entity = models.CharField(max_length=255)
    collections = models.ManyToManyField('apis_metainfo.Collection', blank=True)

#######################################################################
#
#   relation types
#
#######################################################################


#######################################################################
# Person-Relation-Types
#######################################################################


@reversion.register(follow=['relationbaseclass_ptr'])
class PersonPersonRelation(RelationBaseClass):
    """Holds controlled vocabularies relation types of Persons and Persons"""
    pass


@reversion.register(follow=['relationbaseclass_ptr'])
class PersonPlaceRelation(RelationBaseClass):
    """Holds controlled vocabularies relation types of Persons and Places"""
    pass


@reversion.register(follow=['relationbaseclass_ptr'])
class PersonInstitutionRelation(RelationBaseClass):
    """Holds controlled vocabularies relation types of Persons and Persons"""
    pass


@reversion.register(follow=['relationbaseclass_ptr'])
class PersonEventRelation(RelationBaseClass):
    """Holds controlled vocabularies relation types of Persons and Events"""
    pass


@reversion.register(follow=['relationbaseclass_ptr'])
class PersonWorkRelation(RelationBaseClass):
    """Holds controlled vocabularies relation types of Persons and Works"""
    pass


#######################################################################
# Institution-Relation-Types
#######################################################################


@reversion.register(follow=['relationbaseclass_ptr'])
class InstitutionEventRelation(RelationBaseClass):
    """Holds controlled vocabularies relation types of Institutions and Events."""
    pass


@reversion.register(follow=['relationbaseclass_ptr'])
class InstitutionPlaceRelation(RelationBaseClass):
    """Holds controlled vocabularies relation types of Institutions and Places."""
    pass


@reversion.register(follow=['relationbaseclass_ptr'])
class InstitutionInstitutionRelation(RelationBaseClass):
    """Holds controlled vocabularies relation types of Institutions and Institutions."""
    pass


@reversion.register(follow=['relationbaseclass_ptr'])
class InstitutionWorkRelation(RelationBaseClass):
    """Holds controlled vocabularies relation types of Institutions and Works."""
    pass

#######################################################################
# Place-Relation-Types
#######################################################################


@reversion.register(follow=['relationbaseclass_ptr'])
class PlacePlaceRelation(RelationBaseClass):
    """Holds controlled vocabularies relation types of Places and Places"""
    pass


@reversion.register(follow=['relationbaseclass_ptr'])
class PlaceEventRelation(RelationBaseClass):
    """Holds controlled vocabularies relation types of Places and Events"""
    pass


@reversion.register(follow=['relationbaseclass_ptr'])
class PlaceWorkRelation(RelationBaseClass):
    """Holds controlled vocabularies relation types of Places and Works"""
    pass


#######################################################################
# Event-Relation-Types
#######################################################################


@reversion.register(follow=['relationbaseclass_ptr'])
class EventEventRelation(RelationBaseClass):
    """Holds controlled vocabularies relation types of Events and Events"""
    pass


@reversion.register(follow=['relationbaseclass_ptr'])
class EventWorkRelation(RelationBaseClass):
    """Holds controlled vocabularies relation types of Events and Works"""
    pass


#######################################################################
# Work-Relation-Types
#######################################################################


@reversion.register(follow=['relationbaseclass_ptr'])
class WorkWorkRelation(RelationBaseClass):
    """Holds controlled vocabularies relation types of Works and Works"""
    pass



# -*- coding: utf-8 -*-
from django.shortcuts import render, redirect, get_object_or_404
from django.urls import reverse_lazy
from django.contrib.auth.decorators import login_required, user_passes_test
from django.contrib.auth.mixins import UserPassesTestMixin
from django.core.exceptions import FieldError
from django.utils.decorators import method_decorator
from django.urls import reverse
from django.views.generic.edit import DeleteView
from django.views import generic
from django.conf import settings
from django.contrib.contenttypes.models import ContentType
# from reversion import revisions as reversion
from reversion.models import Version
from django.http import HttpResponse
from django.http import HttpResponseRedirect
from django.db.models import Q
from django_tables2 import SingleTableView
from django_tables2 import RequestConfig
from reversion_compare.views import HistoryCompareDetailView
import reversion
from django_tables2.export.views import ExportMixin

from apis_core.helper_functions.utils import access_for_all

from .models import Person, Place, Institution, Event, Work
from .forms import (
    FullTextForm, SearchForm, GenericFilterFormHelper,
    NetworkVizFilterForm, PersonResolveUriForm,
    get_entities_form, GenericEntitiesStanbolForm
)

from apis_core.apis_vocabularies.models import LabelType
from apis_core.apis_metainfo.models import Uri, UriCandidate, TempEntityClass, Text
from apis_core.helper_functions.stanbolQueries import retrieve_obj
from apis_core.helper_functions.RDFparsers import GenericRDFParser
from apis_core.apis_labels.models import Label
from .tables import (
    PersonTable, PlaceTable, InstitutionTable, EventTable, WorkTable,
    get_entities_table
)
from .filters import (
    PersonListFilter, PlaceListFilter, InstitutionListFilter, EventListFilter, WorkListFilter,
    get_generic_list_filter
)

from apis_core.helper_functions.utils import access_for_all, access_for_all_function


import json

if 'apis_highlighter' in settings.INSTALLED_APPS:
    from apis_highlighter.forms import SelectAnnotationProject, SelectAnnotatorAgreement
    from apis_core.helper_functions.highlighter import highlight_text

if 'charts' in settings.INSTALLED_APPS:
    from charts.models import ChartConfig
    from charts.views import create_payload
############################################################################
############################################################################
#
#   Helper Functions
#
############################################################################
############################################################################


@user_passes_test(access_for_all_function)
def set_session_variables(request):
    ann_proj_pk = request.GET.get('project', None)
    types = request.GET.getlist('types', None)
    users_show = request.GET.getlist('users_show', None)
    edit_views = request.GET.get('edit_views', False)
    if types:
        request.session['entity_types_highlighter'] = types
    if users_show:
        request.session['users_show_highlighter'] = users_show
    if ann_proj_pk:
        request.session['annotation_project'] = ann_proj_pk
    if edit_views:
        if edit_views != 'false':
            request.session['edit_views'] = True
    return request


@user_passes_test(access_for_all_function)
def get_highlighted_texts(request, instance):
    if 'apis_highlighter' in settings.INSTALLED_APPS:
        set_ann_proj = request.session.get('annotation_project', 1)
        entity_types_highlighter = request.session.get('entity_types_highlighter', None)
        users_show = request.session.get('users_show_highlighter', None)
        object_texts = [{'text': highlight_text(
            x,
            set_ann_proj=set_ann_proj,
            types=entity_types_highlighter,
            users_show=users_show).strip(),
                         'id': x.pk,
                         'kind': x.kind} for x in Text.objects.filter(tempentityclass=instance)]
        ann_proj_form = SelectAnnotationProject(
            set_ann_proj=set_ann_proj,
            entity_types_highlighter=entity_types_highlighter,
            users_show_highlighter=users_show)
        return object_texts, ann_proj_form
    else:
        object_texts = [{
            'text': x.text,
            'id': x.pk,
            'kind': x.kind
        } for x in Text.objects.filter(tempentityclass=instance)]
        return object_texts, False

############################################################################
############################################################################
#
#   GenericViews
#
############################################################################
############################################################################


class GenericListViewNew(UserPassesTestMixin, ExportMixin, SingleTableView):
    formhelper_class = GenericFilterFormHelper
    context_filter_name = 'filter'
    paginate_by = 25
    template_name = getattr(settings, 'APIS_LIST_VIEW_TEMPLATE', 'apis_entities/generic_list.html')
    login_url = '/accounts/login/'

    def get_model(self):
        model = ContentType.objects.get(
            app_label__startswith='apis_', model=self.entity.lower()
        ).model_class()
        return model

    def test_func(self):
        access = access_for_all(self, viewtype="list")
        if access:
            self.request = set_session_variables(self.request)
        return access

    def get_queryset(self, **kwargs):
        self.entity = self.kwargs.get('entity')
        qs = ContentType.objects.get(
            app_label__startswith='apis_', model=self.entity.lower()
        ).model_class().objects.all()
        self.filter = get_generic_list_filter(self.entity.title())(self.request.GET, queryset=qs)
        self.filter.form.helper = self.formhelper_class()
        return self.filter.qs

    def get_table(self, **kwargs):
        session = getattr(self.request, 'session', False)
        if session:
            edit_v = self.request.session.get('edit_views', False)
        else:
            edit_v = False
        self.table_class = get_entities_table(self.entity.title(), edit_v)
        table = super(GenericListViewNew, self).get_table()
        RequestConfig(self.request, paginate={
            'page': 1, 'per_page': self.paginate_by}).configure(table)
        return table

    def get_context_data(self, **kwargs):
        model = self.get_model()
        context = super(GenericListViewNew, self).get_context_data()
        context[self.context_filter_name] = self.filter
        context['entity'] = self.entity
        context['entity_create_stanbol'] = GenericEntitiesStanbolForm(self.entity)
        if 'browsing' in settings.INSTALLED_APPS:
            from browsing.models import BrowsConf
            context['conf_items'] = list(
                BrowsConf.objects.filter(model_name=self.entity)
                .values_list('field_path', 'label')
            )
        context['docstring'] = "{}".format(model.__doc__)
        if model._meta.verbose_name_plural:
            context['class_name'] = "{}".format(model._meta.verbose_name.title())
        else:
            if model.__name__.endswith('s'):
                context['class_name'] = "{}".format(model.__name__)
            else:
                context['class_name'] = "{}s".format(model.__name__)
        try:
            context['get_arche_dump'] = model.get_arche_dump()
        except AttributeError:
            context['get_arche_dump'] = None
        try:
            context['create_view_link'] = model.get_createview_url()
        except AttributeError:
            context['create_view_link'] = None
        if 'charts' in settings.INSTALLED_APPS:
            app_label = model._meta.app_label
            filtered_objs = ChartConfig.objects.filter(
                model_name=model.__name__.lower(),
                app_name=app_label
            )
            context['vis_list'] = filtered_objs
            context['property_name'] = self.request.GET.get('property')
            context['charttype'] = self.request.GET.get('charttype')
            if context['charttype'] and context['property_name']:
                qs = self.get_queryset()
                chartdata = create_payload(
                    context['entity'],
                    context['property_name'],
                    context['charttype'],
                    qs,
                    app_label=app_label
                )
                context = dict(context, **chartdata)
        return context

    def render_to_response(self, context, **kwargs):
        download = self.request.GET.get('sep', None)
        if download and 'browsing' in settings.INSTALLED_APPS:
            import datetime
            import time
            import pandas as pd
            sep = self.request.GET.get('sep', ',')
            timestamp = datetime.datetime.fromtimestamp(time.time()).strftime('%Y-%m-%d-%H-%M-%S')
            filename = "export_{}".format(timestamp)
            response = HttpResponse(content_type='text/csv')
            if context['conf_items']:
                conf_items = context['conf_items']
                print(dir(self))
                try:
                    df = pd.DataFrame(
                        list(
                            self.get_queryset().values_list(*[x[0] for x in conf_items])
                        ),
                        columns=[x[1] for x in conf_items]
                    )
                except AssertionError:
                    response['Content-Disposition'] = 'attachment; filename="{}.csv"'.format(
                        filename
                    )
                    return response
            else:
                response['Content-Disposition'] = 'attachment; filename="{}.csv"'.format(filename)
                return response
            if sep == "comma":
                df.to_csv(response, sep=',', index=False)
            elif sep == "semicolon":
                df.to_csv(response, sep=';', index=False)
            elif sep == "tab":
                df.to_csv(response, sep='\t', index=False)
            else:
                df.to_csv(response, sep=',', index=False)
            response['Content-Disposition'] = 'attachment; filename="{}.csv"'.format(filename)
            return response
        else:
            response = super(GenericListViewNew, self).render_to_response(context)
            return response


############################################################################
############################################################################
#
#   OtherViews
#
############################################################################
############################################################################

@login_required
def getGeoJson(request):
    '''Used to retrieve GeoJsons for single objects'''
    # if request.is_ajax():
    pk_obj = request.GET.get("object_id")
    instance = get_object_or_404(Place, pk=pk_obj)
    uria = Uri.objects.filter(entity=instance)
    urib = UriCandidate.objects.filter(entity=instance)
    if urib.count() > 0:
        uric = urib
    elif uria.count() > 0:
        uric = uria
        add_info = ""
    lst_json = []
    if uric.count() > 0 and not instance.status.startswith('distinct'):
        for x in uric:
            o = retrieve_obj(x.uri)
            if o:
                url_r = reverse_lazy('apis_entities:resolve_ambigue_place', kwargs={'pk': str(instance.pk), 'uri': o['representation']['id'][7:]})
                select_text = "<a href='{}'>Select this URI</a>".format(url_r)
                try:
                    add_info = "<b>Confidence:</b> {}<br/><b>Feature:</b> <a href='{}'>{}</a>".format(x.confidence, x.uri, x.uri)
                except:
                    add_info = "<b>Confidence:</b>no value provided <br/><b>Feature:</b> <a href='{}'>{}</a>".format(x.uri, x.uri)
                r = {"geometry": {
                        "type": "Point",
                        "coordinates": [
                            float(
                                o['representation']
                                ['http://www.w3.org/2003/01/geo/wgs84_pos#long'][0]['value']),
                            float(
                                o['representation']
                                ['http://www.w3.org/2003/01/geo/wgs84_pos#lat'][0]['value'])
                        ]
                        },
                    "type": "Feature",
                    "properties": {
                        "popupContent": "<b>ÖBL name:</b> %s<br/><b>Geonames:</b> %s<br/>%s<br/>%s" % (instance.name, o['representation']['http://www.geonames.org/ontology#name'][0]['value'], select_text, add_info)
                    },
                    "id": x.pk
                    }
                lst_json.append(r)
    elif instance.lat is not None and instance.lng is not None:
        r = {"geometry": {
            "type": "Point",
            "coordinates": [
                instance.lng,
                instance.lat
            ]
        },
            "type": "Feature",
            "properties": {
                "popupContent": "<b>Name:</b> %s<br/>" % (instance.name)
            },
            "id": instance.pk
        }
        lst_json.append(r)

    return HttpResponse(json.dumps(lst_json), content_type='application/json')


@login_required
def getGeoJsonList(request):
    '''Used to retrieve a list of GeoJsons. To generate the list the kind of connection
    and the connected entity is needed'''
    relation = ContentType.objects.get(app_label='apis_relations', model=request.GET.get("relation")).model_class()
    #relation_type = request.GET.get("relation_type")
    objects = relation.objects.filter(
            related_place__status='distinct').select_related('related_person', 'related_place', 'relation_type')
    lst_json = []
    for x in objects:
        pers_url = x.related_person.get_absolute_url()
        place_url = x.related_place.get_absolute_url()
        r = {"geometry": {
                        "type": "Point",
                        "coordinates": [x.related_place.lng, x.related_place.lat]
                        },
                    "type": "Feature",
                    "relation_type": x.relation_type.name,
                    "properties": {
                        "popupContent": "<b>Person:</b> <a href='%s'>%s</a><br/><b>Connection:</b> %s<br/><b>Place:</b> <a href='%s'>%s</a>" % (pers_url, x.related_person, x.relation_type, place_url, x.related_place)
                    },
                    "id": x.pk
                    }
        lst_json.append(r)
    return HttpResponse(json.dumps(lst_json), content_type='application/json')


@login_required
def getNetJsonList(request):
    '''Used to retrieve a Json to draw a network'''
    relation = ContentType.objects.get(app_label='apis_relations', model='PersonPlace').model_class()
    objects = relation.objects.filter(
            related_place__status='distinct')
    nodes = dict()
    edges = []

    for x in objects:
        if x.related_place.pk not in nodes.keys():
            place_url = reverse_lazy('apis_entities:place_edit', kwargs={'pk': str(x.related_place.pk)})
            tt = "<div class='arrow'></div>\
            <div class='sigma-tooltip-header'>%s</div>\
            <div class='sigma-tooltip-body'>\
            <table>\
                <tr><th>Type</th> <td>%s</td></tr>\
                <tr><th>Entity</th> <td><a href='%s'>Link</a></td></tr>\
            </table>\
            </div>"% (x.related_place.name, 'place', place_url)
            nodes[x.related_place.pk] = {'type': 'place', 'label': x.related_place.name, 'id': str(x.related_place.pk), 'tooltip': tt}
        if x.related_person.pk not in nodes.keys():
            pers_url = reverse_lazy('apis_entities:person_edit', kwargs={'pk': str(x.related_person.pk)})
            tt = "<div class='arrow'></div>\
            <div class='sigma-tooltip-header'>%s</div>\
            <div class='sigma-tooltip-body'>\
            <table>\
                <tr><th>Type</th> <td>%s</td></tr>\
                <tr><th>Entity</th> <td><a href='%s'>Link</a></td></tr>\
            </table>\
            </div>"% (str(x.related_person), 'person', pers_url)
            nodes[x.related_person.pk] = {'type': 'person', 'label': str(x.related_person), 'id': str(x.related_person.pk), 'tooltip': tt}
        edges.append({'source': x.related_person.pk, 'target': x.related_place.pk, 'kind': x.relation_type.name, 'id': str(x.pk)})
    lst_json = {'edges': edges, 'nodes': [nodes[x] for x in nodes.keys()]}

    return HttpResponse(json.dumps(lst_json), content_type='application/json')


@login_required
def getNetJsonListInstitution(request):
    '''Used to retrieve a Json to draw a network'''
    relation = ContentType.objects.get(app_label='apis_relations', model='PersonInstitution').model_class()
    objects = relation.objects.all()
    nodes = dict()
    edges = []

    for x in objects:
        if x.related_institution.pk not in nodes.keys():
            inst_url = reverse_lazy('apis_entities:institution_edit', kwargs={'pk': str(x.related_institution.pk)})
            tt = "<div class='arrow'></div>\
            <div class='sigma-tooltip-header'>%s</div>\
            <div class='sigma-tooltip-body'>\
            <table>\
                <tr><th>Type</th> <td>%s</td></tr>\
                <tr><th>Entity</th> <td><a href='%s'>Link</a></td></tr>\
            </table>\
            </div>"% (x.related_institution.name, 'institution', inst_url)
            nodes[x.related_institution.pk] = {'type': 'institution', 'label': x.related_institution.name, 'id': str(x.related_institution.pk), 'tooltip': tt}
        if x.related_person.pk not in nodes.keys():
            pers_url = reverse_lazy('apis_entities:person_edit', kwargs={'pk': str(x.related_person.pk)})
            tt = "<div class='arrow'></div>\
            <div class='sigma-tooltip-header'>%s</div>\
            <div class='sigma-tooltip-body'>\
            <table>\
                <tr><th>Type</th> <td>%s</td></tr>\
                <tr><th>Entity</th> <td><a href='%s'>Link</a></td></tr>\
            </table>\
            </div>"% (str(x.related_person), 'person', pers_url)
            nodes[x.related_person.pk] = {'type': 'person', 'label': str(x.related_person), 'id': str(x.related_person.pk), 'tooltip': tt}
        edges.append({'source': x.related_person.pk, 'target': x.related_institution.pk, 'kind': x.relation_type.name, 'id': str(x.pk)})
    lst_json = {'edges': edges, 'nodes': [nodes[x] for x in nodes.keys()]}

    return HttpResponse(json.dumps(lst_json), content_type='application/json')


@login_required
def resolve_ambigue_place(request, pk, uri):
    '''Only used to resolve place names.'''
    with reversion.create_revision():
        uri = 'http://'+uri
        entity = Place.objects.get(pk=pk)
        pl_n = GenericRDFParser(uri, kind='Place')
        pl_n_1 = pl_n.save()
        pl_n_1 = pl_n.merge(entity)
        url = pl_n_1.get_absolute_url()
        if pl_n.created:
            pl_n_1.status = 'distinct (manually resolved)'
            pl_n_1.save()
        UriCandidate.objects.filter(entity=entity).delete()
        reversion.set_user(request.user)

    return HttpResponseRedirect(url)


@login_required
def resolve_ambigue_person(request):
    if request.method == "POST":
        form = PersonResolveUriForm(request.POST)
    if form.is_valid():
        pers = form.save()
        return redirect(reverse('apis_entities:person_edit', kwargs={'pk': pers.pk}))
    else:
        print(form)


############################################################################
############################################################################
#
#   VisualizationViews
#
############################################################################
############################################################################

@login_required
def birth_death_map(request):
    return render(request, 'apis_entities/map_list.html')


@login_required
def pers_place_netw(request):
    return render(request, 'apis_entities/network.html')


@login_required
def pers_inst_netw(request):
    return render(request, 'apis_entities/network_institution.html')


@login_required
def generic_network_viz(request):
    if request.method == 'GET':
        form = NetworkVizFilterForm()
        return render(request, 'apis_entities/generic_network_visualization.html',
                      {'form': form})


############################################################################
############################################################################
#
#  Reversion Views
#
############################################################################
############################################################################

class ReversionCompareView(HistoryCompareDetailView):
    template_name = 'apis_entities/compare_base.html'

    def dispatch(self, request, app, kind, pk, *args, **kwargs):
        self.model = ContentType.objects.get(app_label=app, model=kind).model_class()
        return super(ReversionCompareView, self).dispatch(request, *args, **kwargs)



from django.http import Http404
from django_filters.rest_framework import DjangoFilterBackend
from rest_framework import viewsets, generics, filters
from rest_framework.reverse import reverse_lazy
from rest_framework.views import APIView
from rest_framework.response import Response
from rest_framework.exceptions import NotFound, bad_request
from django.db.models import Q
from django.contrib.contenttypes.models import ContentType
from django.conf import settings
from django.shortcuts import redirect
from rest_framework.decorators import api_view

from apis_core.helper_functions.RDFparsers import GenericRDFParser
from .serializers import (
    InstitutionSerializer, PersonSerializer, PlaceSerializer, EventSerializer, WorkSerializer,
    GeoJsonSerializer, NetJsonEdgeSerializer, NetJsonNodeSerializer 
)
from .serializers_generic import EntitySerializer
from .models import Institution, Person, Place, Event, Work
from apis_core.apis_vocabularies.models import VocabsBaseClass
from apis_core.helper_functions.stanbolQueries import find_loc
from apis_core.default_settings.NER_settings import autocomp_settings, stb_base
from apis_core.apis_metainfo.api_renderers import PaginatedCSVRenderer
from apis_core.apis_metainfo.models import TempEntityClass
from apis_core.apis_metainfo.models import Uri
from rest_framework.settings import api_settings
from rest_framework.permissions import DjangoObjectPermissions, AllowAny
from rest_framework.pagination import PageNumberPagination
from rest_framework.parsers import FileUploadParser
from django.urls import reverse

import re
import json
import fnmatch
import os
from io import TextIOWrapper
import requests
from datetime import datetime
# from metainfo.models import TempEntityClass

from . api_renderers import EntityToTEI, EntityToCIDOC


class StandardResultsSetPagination(PageNumberPagination):
    page_size = 25
    page_size_query_param = 'page_size'
    max_page_size = 1000


class GetEntityGeneric(APIView):
    serializer_class = EntitySerializer
    renderer_classes = tuple(
        api_settings.DEFAULT_RENDERER_CLASSES) + (EntityToTEI, EntityToCIDOC)
    if getattr(settings, 'APIS_RENDERERS', None) is not None:
        rend_add = tuple()
        for rd in settings.APIS_RENDERERS:
            rend_mod = __import__(rd)
            for name, cls in rend_mod.__dict__.items():
                rend_add + (cls, )
        renderer_classes += rend_add

    def get_object(self, pk):
        try:
            return TempEntityClass.objects_inheritance.get_subclass(pk=pk)
        except TempEntityClass.DoesNotExist:
            raise Http404

    def get(self, request, pk):
        ent = self.get_object(pk)
        res = EntitySerializer(ent, context={'request': request})
        return Response(res.data)


@api_view(['GET'])
def uri_resolver(request):
    uri = request.query_params.get('uri', None)
    if uri is None:
        raise Http404
    else:
        uri = Uri.objects.get(uri=uri)
        url = reverse('apis_core:apis_api2:GetEntityGeneric', kwargs={'pk': uri.entity_id})
        return redirect(url)


class InstitutionViewSet(viewsets.ModelViewSet):
    """Serialization of the institution class.
    In addition to the institution this view includes related texts and \
    the kind of the institution (separated object)."""
    permission_classes = (DjangoObjectPermissions,)
    queryset = Institution.objects.all()
    serializer_class = InstitutionSerializer
    pagination_class = StandardResultsSetPagination
    filter_backends = (DjangoFilterBackend, filters.SearchFilter)
    depth = 2
    filter_fields = ('name', 'kind__name', 'collection__name')
    search_fields = ('name', )


class PersonViewSet(viewsets.ModelViewSet):
    """Serialization of the Person class.
    In addition to the person this view includes related texts (e.g. biographies), \
    a list of professions (separated objects) and the collection it belongs to."""
    permission_classes = (DjangoObjectPermissions,)
    queryset = Person.objects.all()
    serializer_class = PersonSerializer
    pagination_class = StandardResultsSetPagination
    depth = 2
    filter_backends = (DjangoFilterBackend, filters.SearchFilter)
    filter_fields = (
        'name', 'first_name', 'gender', 'profession__name',
        'collection__name', 'uri__uri'
    )
    search_fields = ('name', 'first_name')


class PlaceViewSet(viewsets.ModelViewSet):
    """Serialization of the Place class.
    In addition to the place this view includes related texts (e.g. from the ÖBL database), \
     and the collection it belongs to."""
    permission_classes = (DjangoObjectPermissions,)
    queryset = Place.objects.all()
    serializer_class = PlaceSerializer
    pagination_class = StandardResultsSetPagination
    filter_backends = (DjangoFilterBackend, filters.SearchFilter)
    depth = 2
    filter_fields = ('name', 'kind__name', 'collection__name',)
    search_fields = ('name', )
    renderer_classes = tuple(api_settings.DEFAULT_RENDERER_CLASSES) + (PaginatedCSVRenderer, )


class EventViewSet(viewsets.ModelViewSet):
    """Serialization of the event class.
    In addition to the event this view includes related texts and
    the kind of the event (separated object)."""
    permission_classes = (DjangoObjectPermissions,)
    queryset = Event.objects.all()
    serializer_class = EventSerializer
    pagination_class = StandardResultsSetPagination
    filter_backends = (DjangoFilterBackend, filters.SearchFilter)
    depth = 2
    filter_fields = ('name', 'kind__name', 'collection__name',)
    search_fields = ('name', )


class WorkViewSet(viewsets.ModelViewSet):
    """Serialization of the work class.
    In addition to the work this view includes related texts and
    the kind of the work (separated object)."""
    permission_classes = (DjangoObjectPermissions,)
    queryset = Work.objects.all()
    serializer_class = WorkSerializer
    pagination_class = StandardResultsSetPagination
    filter_backends = (DjangoFilterBackend, filters.SearchFilter)
    depth = 2
    filter_fields = ('name', 'kind__name', 'collection__name',)
    search_fields = ('name', )


class PlaceGeoJsonViewSet(viewsets.ViewSet):

    def list(self, request):
        queryset = request.query_params.get('q', None)
        diff = re.match(r'^([^\[]*)\[(\d+)\]', queryset)
        match_url = re.search(r'geonames[^0-9]+([0-9]+)', queryset)
        res = False
        if match_url:
            for u in autocomp_settings['Place']:
                if 'url' in u.keys():
                    site = re.search('site/([^/]+)/', u['url'])
                    if site:
                        params = {'id': 'http://sws.geonames.org/{}/'.format(match_url.group(1))}
                        headers = {'Content-Type': 'application/json'}
                        w = requests.get(
                            stb_base+site.group(1)+'/entity', params=params, headers=headers
                        )
                        if w.status_code == 200:
                            if 'representation' in w.json().keys():
                                res = [True, w.json()['representation']]
                                break
        elif diff and queryset:
            queryset = diff.group(1)
            res = find_loc(queryset.split(','), dec_diff=int(diff.group(2)))
        elif queryset:
            res = find_loc(queryset.split(','))
        else:
            res = False, False
        if res[1]:
            if type(res[1]) is not list:
                res = (res[0], [res[1], ])
            serializer = GeoJsonSerializer(res[1], many=True, context={
                'p_pk': request.query_params.get('p_pk', None)}
            )
        else:
            serializer = GeoJsonSerializer(
                [], many=True, context={'p_pk': request.query_params.get('p_pk', None)}
            )
        return Response(serializer.data)


class NetJsonViewSet(viewsets.ViewSet):

    def create(self, request):
        rel = request.data['select_relation']
        if 'select_kind' in request.data.keys():
            kind = request.data['select_kind']
        else:
            kind = None
        if 'annotation_proj' in request.data.keys():
            annotation_proj = request.data['annotation_proj']
        else:
            annotation_proj = None
        if 'ann_include_all' in request.data.keys():
            ann_include_all = True
        else:
            ann_include_all = False
        q = ContentType.objects.get(
            app_label='apis_relations', model=''.join(rel.split('-'))).model_class()
        rel_match = re.match(r'([A-Z][a-z]+)([A-Z][a-z]+$)', rel)
        lst_nodes = []
        rel_a = 'related_' + rel.split('-')[0]
        rel_b = 'related_' + rel.split('-')[1]
        q_dict = {}
        q_list = None
        if rel_a == rel_b:
            rel_a += 'A'
            rel_b += 'B'
        if 'search_source' in request.data.keys():
            source = request.data['search_source']
            if source.startswith('cl:'):
                if q_list is not None:
                    q_list.append(Q(**{rel_a+'__collection__id': int(source[3:])})
                                  | Q(**{rel_b+'__collection__id': int(source[3:])}))
                else:
                    q_dict[rel_a+'__collection__id'] = int(source[3:])
            else:
                if q_list is not None:
                    q_list.append(
                        Q(**{rel_a + '_id': int(source)}) | Q(**{rel_b + '_id': int(source)})
                    )
                else:
                    q_dict[rel_a + '_id'] = int(source)
        elif'search_source-autocomplete' in request.data.keys():
            source = request.data['search_source-autocomplete']
            if q_list is not None:
                q_list.append(
                    Q(**{rel_a + '__name__icontains': source}) | Q(
                        **{rel_b + '__name__icontains': source}
                    )
                )
            else:
                q_dict[rel_a + '__name__icontains'] = source
        if 'search_target' in request.data.keys():
            target = request.data['search_target']
            if target.startswith('cl:'):
                if q_list is not None:
                    q_list.append(Q(**{rel_b+'__collection__id': int(target[3:])})
                                  | Q(**{rel_a+'__collection__id': int(target[3:])}))
                else:
                    q_dict[rel_b+'__collection__id'] = int(target[3:])
            else:
                if q_list is not None:
                    q_list.append(
                        Q(**{rel_a + '_id': int(target)}) | Q(**{rel_b + '_id': int(target)})
                    )
                else:
                    q_dict[rel_b + '_id'] = target
        elif 'search_target-autocomplete' in request.data.keys():
            if len(request.data['search_target-autocomplete']) > 0:
                target = request.data['search_target-autocomplete']
                if q_list is not None:
                    q_list.append(
                        Q(**{rel_a + '__name__icontains': target}) | Q(
                            **{rel_b + '__name__icontains': target})
                        )
                else:
                    q_dict[rel_b + '__name__icontains'] = target
        if kind:
            kind_ids = [int(kind.strip())]
            kind_ids_res = [int(kind.strip())]
            while len(kind_ids) > 0:
                id_a = kind_ids.pop()
                ids_b = VocabsBaseClass.objects.filter(
                    parent_class_id=id_a).values_list('pk', flat=True)
                kind_ids.extend(ids_b)
                kind_ids_res.extend(ids_b)
            q_dict['relation_type_id__in'] = kind_ids_res
        if 'start_date' in request.data.keys():
            if len(request.data['start_date']) > 0:
                start_date_d = datetime.strptime(request.data['start_date'], '%d.%m.%Y')
                print(start_date_d)
                q_dict['end_date__gte'] = start_date_d
        if 'end_date' in request.data.keys():
            if len(request.data['end_date']) > 0:
                end_date_d = datetime.strptime(request.data['end_date'], '%d.%m.%Y')
                print(end_date_d)
                q_dict['start_date__lte'] = end_date_d
        if annotation_proj:
            annProj_filter = {'ann_proj': annotation_proj}
            if not ann_include_all:
                annProj_filter['include_all'] = False
            sr1 = q.annotation_links.filter_ann_proj(
                **annProj_filter).filter(**q_dict).select_related(rel_a, rel_b)
        else:
            sr1 = q.objects.filter(**q_dict)
        if q_list is not None:
            sr1 = sr1.filter(q_list[0])
        for node in sr1:
            if getattr(node, rel_a) not in lst_nodes:
                lst_nodes.append(getattr(node, rel_a))
            if getattr(node, rel_b) not in lst_nodes:
                lst_nodes.append(getattr(node, rel_b))
        sr1_1 = NetJsonEdgeSerializer(sr1, many=True)
        sr1_2 = NetJsonNodeSerializer(lst_nodes, many=True)
        res = {'nodes': sr1_2.data, 'edges': sr1_1.data}
        return Response(res)


class SaveNetworkFiles(APIView):

    def post(self, request, format=None):
        file_name = request.data['file_name']
        file_name_list = file_name.split('.')
        data = request.data['file']
        nmb = False
        for file in os.listdir('downloads/'):
            print(file)
            if fnmatch.fnmatch(
                file, file_name_list[0]+'_*.'+file_name_list[1]
            ) or file == file_name:
                if nmb:
                    nmb += 1
                else:
                    nmb = 1
        if nmb:
            file_name = file_name_list[0]+'_'+str(nmb)+'.'+file_name_list[1]
        with open('downloads/'+file_name, 'wb') as fw:
            data = data.replace('"edges"', '"links"', 1)
            fw.write(data.encode("utf-8"))
        res = {'test': True, 'file_name': '/downloads/'+file_name}
        return Response(res)


class ResolveAbbreviations(APIView):
    parser_classes = (FileUploadParser,)

    def put(self, request, filename, format=None):
        file_obj = request.data['file']
        txt = TextIOWrapper(file_obj, encoding='utf8')
        return Response(status=204)


class GetOrCreateEntity(APIView):

    def get(self, request):
        entity = request.query_params.get('entity2', None)
        uri = request.query_params.get('uri', None)
        ent = GenericRDFParser(uri, entity.title()).get_or_create()
        res = {
            'id': ent.pk,
            'url': reverse_lazy(
                'apis:apis_entities:generic_entities_edit_view',
                request=request,
                kwargs={'pk': ent.pk, 'entity': entity})
            }
        return Response(res)
